#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::MapperMWSProduct
# state        public
#----------------------------------------------------------------------------------------
# description  Maps the MWSProduct webservice from Amazon to a data transfer object
#----------------------------------------------------------------------------------------
#========================================================================================
package DE_EPAGES::Amazon::API::Object::MapperMWSProduct;
use strict;

our $STDURL    = 'http://mws.amazonservices.com/schema/Products/2011-10-01/default.xsd';
our $ANSURL    = 'http://mws.amazonservices.com/schema/Products/2011-10-01';

use DE_EPAGES::Amazon::API::DTO::MWSProduct;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use Storable;
use XML::LibXML;
use XML::LibXML::XPathContext;
use JSON qw ( from_json );


#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# input        $stdNameSpace | standart xml namespace | String
# input        $ansNameSpace | amazon xml namespace   | String
#----------------------------------------------------------------------------------------
# syntax       my $Object = new('http://standart/namespace/URL','http://amazon/namespace/URL');
#----------------------------------------------------------------------------------------
# description  constructor for MWSProduct webservice. Can receice two xml namespaces 
#               If not it will use the default URLs.
#----------------------------------------------------------------------------------------
# return       $self | the Object | object
#========================================================================================
sub new {
    my $class = shift;
    my ($stdNameSpace,$ansNameSpace)  = @_;
    my $self  = bless DE_EPAGES::Core::API::Object::BaseObject->new(), $class;
    $self->{'STDURL'} = $stdNameSpace?$stdNameSpace:$STDURL;
    $self->{'ANSURL'} = $ansNameSpace?$ansNameSpace:$ANSURL;
    return $self;
}

#========================================================================================
# function     initLibXML
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $LibXMLHandler = initLibXML( $XML );
#----------------------------------------------------------------------------------------
# input        $XML | a xml as a String | String
#----------------------------------------------------------------------------------------
# description  checks if the incomming string is a xml and initiated LibXML  
#----------------------------------------------------------------------------------------
# return       $xpc | a LibXML handler | object
#========================================================================================
sub initLibXML {
    my $self = shift;
    my ($XML) = @_;
    if ( (not defined $XML) && (not $XML =~ m/^<?xml version="1.0"/) ){
        Error('XMLFileError', {
            'Reason'  =>, 'incomming string is not defined or not in XML structure',
            'Store'    => LoadRootObject()->get('StoreName'),
        });
    }
    my $dom = XML::LibXML->load_xml( string => $XML );
    my $xpc = XML::LibXML::XPathContext->new( $dom );
    $xpc->registerNs( 'std' => $self->{'STDURL'} );# set name space one
    $xpc->registerNs( 'ans' => $self->{'ANSURL'} );# set name space two
    return $xpc;
}

#========================================================================================
# function     _ErrorMessageCheck
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ErrorMessageCheck( $XML,$xpc );
#----------------------------------------------------------------------------------------
# input        $XML | a xml as a String | String
# input        $xpc | a LibXML handler | object
#----------------------------------------------------------------------------------------
# description  checks if the XML contains an error message.
#========================================================================================
sub _ErrorMessageCheck{
    my $self = shift;
    my ($XML,$xpc) = @_;
    if ($XML =~ /ErrorResponse/){
        my $ErrorMessageNode = $xpc->findnodes('//ans:Message')->pop();
        my $ErrorMessage = $ErrorMessageNode?$ErrorMessageNode->textContent():'unknown Error in mapXMLToDTO';
        Error('AWSRequestError', {
            'Reason'  =>, $ErrorMessage,
        });
    }
}

#========================================================================================
# function     _GetMappingList
# state        private
#----------------------------------------------------------------------------------------
# syntax       my $MappingList = _GetMappingList();
#----------------------------------------------------------------------------------------
# description  contains a mapping list between the Amazons service MWSProduct xml nodes 
#               and the epages DTO sub names.
#               SubNameInDTO             => 'XMLNodeName'
#----------------------------------------------------------------------------------------
# return       $_ | a Hash | refhash
#========================================================================================

sub _GetMappingList{
    return {
            'Brand'                      => './std:Brand',
            'Binding'                    => './std:Binding',
            'Department'                 => './std:Department',
            'Color'                      => './std:Color',
            'DimensionHeight'            => './std:ItemDimensions/std:Height',
            'DimensionLength'            => './std:ItemDimensions/std:Length',
            'DimensionWidth'             => './std:ItemDimensions/std:Width',
            'DimensionWeight'            => './std:ItemDimensions/std:Weight',
            'DimensionLenghtUnit'        => './std:ItemDimensions/std:Height/@Units',
            'DimensionWeightUnit'        => './std:ItemDimensions/std:Weight/@Units',
            'Label'                      => './std:Label',
            'ListPrice'                  => './std:ListPrice/std:Amount',
            'ListPriceCurrency'          => './std:ListPrice/std:CurrencyCode',
            'Manufacturer'               => './std:Manufacturer',
            'MaterialType'               => './std:MaterialType',
            'Model'                      => './std:Model',
            'PackageDimensionsHeight'    => './std:PackageDimensions/std:Height',
            'PackageDimensionsLength'    => './std:PackageDimensions/std:Length',
            'PackageDimensionsWidth'     => './std:PackageDimensions/std:Width',
            'PackageDimensionsWeight'    => './std:PackageDimensions/std:Weight',
            'PackageDimensionLenghtUnit' => './std:PackageDimensions/std:Height/@Units',
            'PackageDimensionWeightUnit' => './std:PackageDimensions/std:Weight/@Units',
            'PartNumber'                 => './std:PartNumber',
            'ProductGroup'               => './std:ProductGroup',
            'ProductTypeName'            => './std:ProductTypeName',
            'ReleaseDate'                => './std:ReleaseDate',
            'Publisher'                  => './std:Publisher',
            'Size'                       => './std:Size',
            'ImageURL'                   => './std:SmallImage/std:URL',
            'ImageHeight'                => './std:SmallImage/std:Height',
            'ImageWidth'                 => './std:SmallImage/std:Width',
            'Studio'                     => './std:Studio',
            'Title'                      => './std:Title',
    };
}

#========================================================================================
# function     _MapValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       _MapValues( $xpc,$DTOObject,$MappingList );
#----------------------------------------------------------------------------------------
# input        $xpc | a LibXML handler | object
# input        $DTOObject | a Object ref to the DTO | object_ref
# input        $MappingList | a List with pairs of sub names and XML node names | hash_ref
#----------------------------------------------------------------------------------------
# description  It take the XMLNodeName from the mapping list and find the Node in the XML 
#               which handover via the LibXML handler. If the SubName is defined in the DTO
#               the content of the XMLNode will stored in the DTO.
#========================================================================================

sub _MapValues{
    my $self = shift;
    my ($xpc,$DTOObject,$MappingList) = @_;
    while ( my ($DTOSubName, $XMLNodeName) = each(%{$MappingList}) ) {
        if ($DTOObject->can($DTOSubName)){
            my $Node = $xpc->findnodes($XMLNodeName)->pop();
            $DTOObject->$DTOSubName($Node->textContent()) if defined $Node;
        }
        else{
            Error('MappingError', {
            'Reason'  =>, 'No DTO Sub Name: "'. $DTOSubName.'"',
        });
        }
    }
}

#========================================================================================
# function     my $ListsOfDTOs = mapJSONToDTO
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ListsOfDTOs = mapJSONToDTO( $XML, $ShopAlias, $ShopURL );
#----------------------------------------------------------------------------------------
# input        $ResponseContent | utf8 decoded response content | String
# input        $ShopAlias | Alias of the Shop | String
# input        $ShopURL | the URL of the shop to build the Link to the Amazon product | String
#----------------------------------------------------------------------------------------
# description  maps between the Amazons service MWSProduct json response and the epages DTO
#----------------------------------------------------------------------------------------
# return       \@DTOList | a list of DTOs | hash_ref
#========================================================================================
sub mapJSONToDTO {
    my $self = shift;
    my ($ResponseContent, $ShopAlias, $ShopURL) = @_;
    if (not defined $ShopURL){
        Error('URLError', {
            'Reason'   => 'incomming URL string is not defined',
            'ShopName' => $ShopAlias,
            'Store'    => LoadRootObject()->get('StoreName'),
        });
    }

    my $hProducts;
    eval {
        $hProducts = from_json($ResponseContent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Warning($Error);
    }

    $hProducts = $hProducts->{'items'};

    my @DTOList;
    foreach my $hProduct (@$hProducts) {
        my $DTOObject = DE_EPAGES::Amazon::API::DTO::MWSProduct->new();

        # ASIN
        $DTOObject->ASIN($hProduct->{'asin'}) if defined $hProduct->{'asin'};

        # URL
        $DTOObject->set( 'URL' => ($ShopURL.'/dp/'.$DTOObject->ASIN));

        # Brand
        $DTOObject->Brand($hProduct->{'summaries'}[0]->{'brand'}) if defined $hProduct->{'summaries'}[0]->{'brand'};

        # Department
        $DTOObject->Department($hProduct->{'attributes'}->{'department'}[0]->{'value'}) if defined $hProduct->{'attributes'}{'department'}[0]->{'value'};

        # Color
        $DTOObject->Color($hProduct->{'summaries'}[0]->{'color'}) if defined $hProduct->{'summaries'}[0]->{'color'};

        my @Temp;
        foreach my $FeatureNode (@{ $hProduct->{'attributes'}->{'special_feature'} }){
            push (@Temp, $FeatureNode->{'value'}) if defined $FeatureNode->{'value'};
        }
        # Feature
        $DTOObject->Feature(join(' - ',@Temp));

        # ImageHeight
        $DTOObject->ImageHeight(75);
        # ImageWidth
        $DTOObject->ImageWidth(75);
        # ImageURL
        $DTOObject->ImageURL($hProduct->{'images'}[0]->{'images'}[0]->{'link'}) if defined $hProduct->{'images'}[0]->{'images'}[0]->{'link'};

        # DimensionHeight
        $DTOObject->DimensionHeight($hProduct->{'dimensions'}[0]->{'item'}{'height'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'item'}{'height'}->{'value'};
        # DimensionWidth
        $DTOObject->DimensionWidth($hProduct->{'dimensions'}[0]->{'item'}{'width'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'item'}{'width'}->{'value'};
        # DimensionLength
        $DTOObject->DimensionLength($hProduct->{'dimensions'}[0]->{'item'}{'length'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'item'}{'length'}->{'value'};
        # DimensionLenghtUnit
        $DTOObject->DimensionLenghtUnit($hProduct->{'dimensions'}[0]->{'item'}{'length'}->{'unit'}) if defined $hProduct->{'dimensions'}[0]->{'item'}{'height'}->{'unit'};
        # DimensionWeight
        $DTOObject->DimensionWeight($hProduct->{'dimensions'}[0]->{'item'}{'weight'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'item'}{'weight'}->{'value'};
        # DimensionWeightUnit
        $DTOObject->DimensionWeightUnit($hProduct->{'dimensions'}[0]->{'item'}{'weight'}->{'unit'}) if defined $hProduct->{'dimensions'}[0]->{'item'}{'weight'}->{'unit'};

        # Manufacturer
        $DTOObject->Manufacturer($hProduct->{'summaries'}[0]->{'manufacturer'}) if defined $hProduct->{'summaries'}[0]->{'manufacturer'};
        # Label
        $DTOObject->Label($hProduct->{'summaries'}[0]->{'manufacturer'}) if defined $hProduct->{'summaries'}[0]->{'manufacturer'};

        # ListPrice
        $DTOObject->ListPrice($hProduct->{'attributes'}->{'list_price'}[0]->{'value_with_tax'}) if defined $hProduct->{'attributes'}->{'list_price'}[0]->{'value_with_tax'};
        # ListPriceCurrency
        $DTOObject->ListPriceCurrency($hProduct->{'attributes'}->{'list_price'}[0]->{'currency'}) if defined $hProduct->{'attributes'}->{'list_price'}[0]->{'currency'};

        # MaterialType
        $DTOObject->MaterialType($hProduct->{'attributes'}->{'material'}[0]->{'value'}) if defined $hProduct->{'attributes'}->{'material'}[0]->{'value'};

        # PackageDimensionsHeight
        $DTOObject->PackageDimensionsHeight($hProduct->{'dimensions'}[0]->{'package'}->{'height'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'package'}->{'height'}->{'value'};
        # PackageDimensionsWeight
        $DTOObject->PackageDimensionsWeight($hProduct->{'dimensions'}[0]->{'package'}->{'weight'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'package'}->{'weight'}->{'value'};
        # PackageDimensionWeightUnit
        $DTOObject->PackageDimensionWeightUnit($hProduct->{'dimensions'}[0]->{'package'}->{'weight'}->{'unit'}) if defined $hProduct->{'dimensions'}[0]->{'package'}->{'weight'}->{'unit'};
        # PackageDimensionsLength
        $DTOObject->PackageDimensionsLength($hProduct->{'dimensions'}[0]->{'package'}->{'length'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'package'}->{'length'}->{'value'};
        # PackageDimensionLenghtUnit
        $DTOObject->PackageDimensionLenghtUnit($hProduct->{'dimensions'}[0]->{'package'}->{'length'}->{'unit'}) if defined $hProduct->{'dimensions'}[0]->{'package'}->{'length'}->{'unit'};
        # PackageDimensionsWidth
        $DTOObject->PackageDimensionsWidth($hProduct->{'dimensions'}[0]->{'package'}->{'width'}->{'value'}) if defined $hProduct->{'dimensions'}[0]->{'package'}->{'width'}->{'value'};

        # Size
        $DTOObject->Size($hProduct->{'summaries'}[0]->{'size'}) if defined $hProduct->{'summaries'}[0]->{'size'};

        # PartNumber
        $DTOObject->PartNumber($hProduct->{'summaries'}[0]->{'partnumber'}) if defined $hProduct->{'summaries'}[0]->{'partnumber'};

        # ProductGroup
        $DTOObject->ProductGroup($hProduct->{'summaries'}[0]->{'websiteDisplayGroupName'}) if defined $hProduct->{'summaries'}[0]->{'websiteDisplayGroupName'};

        # ProductTypeName
        $DTOObject->ProductTypeName($hProduct->{'productTypes'}[0]->{'productType'}) if defined $hProduct->{'productType'}[0]->{'productType'};

        # Publisher
        $DTOObject->Publisher($hProduct->{'summaries'}[0]->{'manufacturer'}) if defined $hProduct->{'summaries'}[0]->{'manufacturer'};
        # ReleaseDate
        $DTOObject->ReleaseDate($hProduct->{'attributes'}->{'publication_date'}[0]->{'value'}) if defined $hProduct->{'attributes'}->{'publication_date'}[0]->{'value'};
        # Studio
        $DTOObject->Studio($hProduct->{'summaries'}[0]->{'manufacturer'}) if defined $hProduct->{'summaries'}[0]->{'manufacturer'};

        # Title
        $DTOObject->Title($hProduct->{'summaries'}[0]->{'itemName'}) if defined $hProduct->{'summaries'}[0]->{'itemName'};

        push (@DTOList,$DTOObject);
    }

    return \@DTOList;
}

#========================================================================================
# function     my $ListsOfDTOs = mapXMLToDTO
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ListsOfDTOs = mapXMLToDTO( $XML, $ShopURL );
#----------------------------------------------------------------------------------------
# input        $XML | a xml as a String | String
# input        $ShopURL | the URL of the shop to build the Link to the Amazon product | String
#----------------------------------------------------------------------------------------
# description  maps between the Amazons service MWSProduct xml response and the epages DTO  
#----------------------------------------------------------------------------------------
# return       \@DTOList | a list of DTOs | hash_ref
#========================================================================================
sub mapXMLToDTO {
    my $self = shift;
    my ($XML, $ShopURL) = @_;
    if (not defined $ShopURL ){
        Error('URLError', {
            'Reason'  =>, 'incomming URL string is not defined',
            'Store'    => LoadRootObject()->get('StoreName'),
        });
    }
    my $xpc = $self->initLibXML($XML);
    $self->_ErrorMessageCheck($XML,$xpc);
    my @DTOList;
    my @Products = $xpc->findnodes('//ans:Product');
    for my $Product ( @Products ) {
        my $DTOObject = DE_EPAGES::Amazon::API::DTO::MWSProduct->new();
        my $ASINNode = $xpc->findnodes('ans:Identifiers/ans:MarketplaceASIN/ans:ASIN',$Product)->pop();# get the first element
        # ASIN 
        $DTOObject->ASIN($ASINNode->textContent()) if defined $ASINNode;
        # URL
        $DTOObject->set( 'URL' => ($ShopURL.'/dp/'.$DTOObject->ASIN));
        my $wantedNode = $xpc->findnodes('./ans:AttributeSets/std:ItemAttributes',$Product)->pop();
        if ( $wantedNode ) {
            $xpc->setContextNode($wantedNode);
            $self->_MapValues ($xpc,$DTOObject,_GetMappingList());
            # collecting all features together and turn it into a String.
            my @Temp;
            foreach my $FeatureNode ($xpc->findnodes('./std:Feature')){
                push (@Temp, $FeatureNode->textContent()) if defined $FeatureNode;
            }
            $DTOObject->Feature(join(' - ',@Temp));
            # product variations
            my $Node = $xpc->findnodes('../../ans:Relationships/ans:VariationParent/ans:Identifiers/ans:MarketplaceASIN/ans:ASIN')->pop();
            if (defined $Node){
                $DTOObject->VariationParentASIN($Node->textContent());
                $DTOObject->VariationsProducts(0);
            }
            else {
                $DTOObject->VariationParentASIN(0);
                $DTOObject->VariationsProducts(_GetSubProducts($self,$xpc));
            }
            push (@DTOList,$DTOObject);
        }
    }
    return \@DTOList;
}

#========================================================================================
# function     _GetSubProducts
# state        private
#----------------------------------------------------------------------------------------
# syntax       _GetSubProducts( $xpc );
#----------------------------------------------------------------------------------------
# input        $xpc | a LibXML handler | object
#----------------------------------------------------------------------------------------
# description  build for all subproducts a hash which contains a assignment hash 
#               and DataSets. 
#----------------------------------------------------------------------------------------
# return       $_ | a hash ref with an assignment hash and all subproduct Values | hash_ref
#========================================================================================

sub _GetSubProducts{
    my $self = shift;
    my ($xpc) = @_;
    my $aAttributesNames = _getSubProductAttributeNames($self,$xpc);
    my @DataSets;
    foreach my $AttributesName (@{$aAttributesNames}){
        push (@DataSets,_BuildDataSets($self,$xpc,$AttributesName));
    }
    return {
            DataSets   => \@DataSets,
            Assignment => _BuildAssignmentHash($self,$xpc),
    };
}

#========================================================================================
# function     _BuildAssignmentHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       _BuildAssignmentHash( $xpc );
#----------------------------------------------------------------------------------------
# input        $xpc | a LibXML handler | object
#----------------------------------------------------------------------------------------
# description  buid a list which maps between subproduct Values and the ASIN 
#----------------------------------------------------------------------------------------
# return       \%AssignmentHash | a mapping hash  | hash_ref
#========================================================================================

sub _BuildAssignmentHash {
    my $self = shift;
    my ( $xpc ) = @_;
    my %AssignmentHash;
    my $aAttributesNames = _getSubProductAttributeNames($self,$xpc);
    my @VariationNodes = $xpc->findnodes('//std:VariationChild');
        if ( ( scalar @VariationNodes ) >= 1 ){
            foreach my $VariationNode (@VariationNodes){
                $xpc->setContextNode($VariationNode);
                my $ASIN = $xpc->findnodes('./ans:Identifiers/ans:MarketplaceASIN/ans:ASIN')->pop();
                if (defined $ASIN){
                    my $DataNodeKey = '';
                    my @DataNodes = $xpc->findnodes( './*[not(*)]' ); # find all nodes without sub nodes
                    foreach my $DataNode (@DataNodes){
                        if (defined $DataNode && (grep {$_ eq $DataNode->nodeName()} @{$aAttributesNames})) {##no critic (ProhibitBooleanGrep)
                            $DataNodeKey .= $DataNode->textContent;
                        }
                    }
                    $DataNodeKey =~ s/\s+//g; # remove all whitespaces
                    $AssignmentHash{$DataNodeKey} = $ASIN->textContent();
                }
            }
        }
    return \%AssignmentHash;
}

#========================================================================================
# function     _BuildDataSets
# state        private
#----------------------------------------------------------------------------------------
# syntax       _BuildDataSets( $xpc,$TagName );
#----------------------------------------------------------------------------------------
# input        $xpc | a LibXML handler | object
# input        $TagName | a Name of a subproduct value | String
#----------------------------------------------------------------------------------------
# description  goes through all subproducts and takes the values to the given TagName.
#----------------------------------------------------------------------------------------
# return       $_ | returns a Hash which contains the Tagname and a list with values | hash_ref
#========================================================================================
sub _BuildDataSets {
    my $self = shift;
    my ( $xpc,$TagName ) = @_;
    my @Attribute;
    my @VariationNodes = $xpc->findnodes('//std:VariationChild');
    if ( ( scalar @VariationNodes ) >= 1 ){
        foreach my $VariationNode (@VariationNodes){
            $xpc->setContextNode($VariationNode);
            my $Node = $VariationNode->getChildrenByTagName($TagName)->pop();
            if (defined $Node && not (grep {$_ eq $Node->textContent()} @Attribute)) { ##no critic (ProhibitMixedBooleanOperators,ProhibitBooleanGrep)
                push (@Attribute,$Node->textContent);
            }
        }
    }
    # remove all XML namespaces like ns2:, std:
    $TagName =~ s/\w+://g;
    return {
        Name => $TagName,
        Data => \@Attribute,
    };
}

#========================================================================================
# function     _getSubProductAttributeNames
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getSubProductAttributeNames( $xpc );
#----------------------------------------------------------------------------------------
# input        $xpc | a LibXML handler | object
#----------------------------------------------------------------------------------------
# description  goes through all subproducts and takes the Tagnames
#----------------------------------------------------------------------------------------
# return       \@ValueNames | a List with tag names | array_ref
#========================================================================================

sub _getSubProductAttributeNames {
    my $self = shift;
    my ($xpc) = @_;
    my @ValueNames;
    my @VariationNodes = $xpc->findnodes('//std:VariationChild');
     if ( ( scalar @VariationNodes ) >= 1 ){
        $xpc->setContextNode( $VariationNodes[0] );
        my @DataNodes = $xpc->findnodes( './*[not(*)]' ); # find all nodes without subnodes
        foreach my $DataNode (@DataNodes){
            push( @ValueNames, $DataNode->nodeName );
        }
     }
    return \@ValueNames;
}

1;
