#========================================================================================
# package      DE_EPAGES::Amazon::Hooks::ShopRating
# state        public
# description  runs when order-related mails are being sent.
#========================================================================================
package DE_EPAGES::Amazon::Hooks::Shop;

use strict;
use DateTime::Duration;
use DE_EPAGES::Amazon::API::Cartridge;
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforeCreateUnityShop.
#               Imports the shop notification for amazon events
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Shop - shop object | object</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Object'};
    # import notifications
    my $Cartridge = DE_EPAGES::Amazon::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/ShopNotifications.xml', $Shop->pathString);

    return;
}

#========================================================================================
# function     OnMBOViewMBO
#----------------------------------------------------------------------------------------
# syntax       OnMBOViewMBO( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook UI_MBOViewMBO.
#               Check for Unity shops if the amazon auth token expires
#               within 30 days and enables the notificaton
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Servlet - servlet object | object</li>
#                   <li>Shop - shop object | object</li>
#                   <li>Form - form object | object</li>
#                   <li>Values - value hash | hash</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnMBOViewMBO {
    my ($hParams) = @_;

    my $Shop = $hParams->{'Shop'};
    my $Servlet = $hParams->{'Servlet'};

    if ($Shop->get('IsUnityRegistered')) {

        my $aShopPortalConfigs = $Shop->get('PortalConfigs');
        return unless scalar(@$aShopPortalConfigs);

        if ($Shop->folder('ShopNotifications')->existsChild('AmazonAuthExpire')) {
            my $AmazonAuthExpireNotification = $Shop->folder('ShopNotifications')->child('AmazonAuthExpire');
            my $Duration = DateTime::Duration->new(days => 30);
            my $Now = GetCurrentDBHandle()->currentDateTime;
            my $NowNormDay = $Now->clone->truncate(to => 'day');
            my $StartWarnDate = $Now + $Duration;

            foreach my $PortalConfig (@$aShopPortalConfigs) {
                if ($PortalConfig->class()->isSubClassOf('AmazonConfig')) {
                    #convert epoch time to DateTime object
                    next unless $PortalConfig->get('SPAPI_RefreshTokenExpiry');
                    my $TokenExpirationDate = DateTime->from_epoch('epoch' => $PortalConfig->get('SPAPI_RefreshTokenExpiry'));

                    #test if we have a token?
                    next unless defined $TokenExpirationDate;

                    # set to start of the day
                    my $TokenExpirationNormDay = $TokenExpirationDate->clone->truncate(to => 'day');

                    # enable amazon shop notification for expired tokens
                    if ($StartWarnDate >= $TokenExpirationNormDay) {
                        $AmazonAuthExpireNotification->addObject($PortalConfig);
                        my $RemainingDays = $TokenExpirationNormDay - $NowNormDay;
                        $Servlet->params("RemainingDays_" . $PortalConfig->id, $RemainingDays->{'days'});
                    }
                }
            }
        }
    }

    return;
}

1;
