#==============================================================================
# package      DE_EPAGES::Amazon::UI::AmazonOffer
# state        private
#==============================================================================
package DE_EPAGES::Amazon::UI::AmazonOffer;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use Readonly;
use DE_EPAGES::Core::API::Log qw ( LogDebug LogError );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Presentation::API::Object::PagerChildren;
use DE_EPAGES::Amazon::API::Constants qw ( %AMAZON_ONLINESTATUS );
use DE_EPAGES::Amazon::API::Object::AmazonConfig qw ( updateBestPrice );
use DE_EPAGES::Amazon::API::Object::PagerMWSProductItems;
use DE_EPAGES::Amazon::API::RemoteAdapterFactory;
use DE_EPAGES::Product::API::Product qw (TestUPCEAN);


local $XML::Simple::PREFERRED_PARSER='XML::Parser';

#==============================================================================
# Event handlers
#==============================================================================
sub ViewVariations {
    my $self      = shift;
    my ($Servlet) = @_;
    my $AmazonOffer = $Servlet->object;
    my $PortalConfig = $AmazonOffer->get('PortalConfig');
    my $Pager = DE_EPAGES::Presentation::API::Object::PagerChildren->new({
        'Servlet'  => $Servlet,
        'Object'   => $AmazonOffer,
        'PageSize' => 20,
        'OrderBy'  => $Servlet->form->value('OrderBy')//'Alias'
    });

    $Pager->statistic;
    $PortalConfig->updateBestPrice( $Pager->items( $Pager->get('Page') ) ) if $Pager->get('CountOfPages') > 0;

    return $Pager->page;
}

sub View {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Form      = $Servlet->form;
    my $Offer     = $Servlet->object;
    my $Product   = $Offer->get('Product');

    my $PortalConfig = $Offer->get('PortalConfig');

    # reverse sort error messages.
    my @aMessages = reverse sort { $a->id <=> $b->id }
                    grep { $_->get('IsReviewed') == 0 && defined $_->get('ErrorMessage')}
                    @{LoadClassByAlias('AmazonFeedMessage')->getObjects('ByAmazonOfferID', [$Offer->id])};

    $Servlet->vars('ErrorMessages',[map {$_->tleHash} @aMessages]);

    return if (scalar @{$Product->folder('SubProducts')->children});
    my $Remote = DE_EPAGES::Amazon::API::RemoteAdapterFactory::NewRemoteMWSProduct( $PortalConfig );

    my $SearchString = defined $Form->value('SearchString') ? $Form->value('SearchString') : $Form->value('WildSearchString');
    my $BrandName = defined $Form->value('SearchStringBrand') ? $Form->value('SearchStringBrand') : undef;
    my $EanAsin = defined $Form->value('SearchEanAsin') ? $Form->value('SearchEanAsin') : undef;

    $Servlet->vars('SearchString', $SearchString);
    my $Pager = eval {
        DE_EPAGES::Amazon::API::Object::PagerMWSProductItems->new({
            'Servlet'       => $Servlet,
            'Remote'        => $Remote,
            'Parameters'    => {
                'ASINs'         => $Offer->get('ASIN') || $EanAsin  ,
                'UPCEAN'        => $Product->get('UPCEAN'),
                'ProductName'   => $Product->getWithParent('Name'),
                'BrandName'   => $Product->getWithParent('Manufacturer'),
            },
            'UriParameters' => {
                'SearchString' => $SearchString,
                'BrandName'    => $BrandName,
                'EanAsin'      => $Offer->get('ASIN') || $EanAsin,
            }
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogDebug( 'Error:', $Error);
        $Servlet->vars('MWSProductRequestError',$Error->shortMessage);
        if ($Error->shortMessage =~ m/does not have access to the given marketplace/){
            $Servlet->vars('BadMerchantID',$Error->shortMessage);
        }
        return {};
    }

    my $BadEanAsin = $Pager->get('BadEanAsin');
    $Servlet->vars('BadEanAsin', $BadEanAsin);

    my @OfferArray = ($Offer);
    $PortalConfig->updateBestPrice( \@OfferArray  );

    return $Pager->page;
}

sub Assign {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Offer     = $Servlet->object;
    my $Form      = $Servlet->form;

    my $hFormValues = $Form->form($Offer,'Assign');

    if ($hFormValues->{'ASIN'}) {
        $Offer->set({'ASIN' => $hFormValues->{'ASIN'}});
    }
}

sub Unassign {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Offer     = $Servlet->object;

    $Offer->set({'ASIN' => undef});
}

sub Save {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Offer     = $Servlet->object;
    my $Form      = $Servlet->form;

    if ( $Offer->get('ASIN') ne $Form->value('ASIN') ){
           $self->Assign($Servlet);
    }

    $Form->executeFormError({
            'Name'       => 'ProductNumberTooLong',
            'Reason'     => 'ProductNumberTooLong',
    }) if ( length $Offer->get('Product')->get('Alias') > 40 );

    my $hFormValues = $Servlet->form->form($Offer, 'AmazonOffer');
    $Offer->updateValues($hFormValues);

    if (ExistsError()) {
        my $Error = GetError();
        $Form->executeFormError( {
            'Name'       => 'View',
            'Reason'     => $Error->code,
            'ViewAction' => 'MBO-ViewGeneral',
        } );
    }
}

sub Start {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Form      = $Servlet->form;
    my $Offer     = $Servlet->object;

    $self->Save($Servlet);

    my $PortalConfig = $Offer->get('PortalConfig');
    if (not $PortalConfig->get('IsSettingsComplete')) {
        return;
    }

    my $aWarnings = BlockWarnings( sub {
        eval {
            $Offer->startOffer();
        };
        # failures running startOffer
        if (ExistsError()) {
            my $Error = GetError();
            $Form->executeFormError( {
                'Name'       => 'Start',
                'Reason'     => $Error->code,
                'ViewAction' => 'MBO-ViewGeneral',
            });
        } else {
            $Servlet->vars('Listing_OK', 1);
        }
    } );

    foreach my $Warning (@$aWarnings){
        $Form->mapFormError({'Name'=>'AmazonOffer'}, $Warning);
    }

}

sub End {
    my $self      = shift;
    my ($Servlet) = @_;
    my $Form      = $Servlet->form;
    my $Offer     = $Servlet->object;
    my $PortalConfig = $Offer->get('PortalConfig');

    eval {
        $PortalConfig->hideOffers([$Offer]);
    };

    if (ExistsError()) {
            my $Error = GetError();
            LogError($Error);
            $Form->executeFormError( {
                'Name'       => 'End',
                'Reason'     => $Error->code,
                'ViewAction' => 'MBO-ViewGeneral',
            } );
        }
}

sub EndSubOffers {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $Form         = $Servlet->form;
    my $Offer        = $Servlet->object;
    my $PortalConfig = $Offer->get('PortalConfig');
    my $hValues      = $Form->form($Offer, 'ListedObjects');
    my $Shop         = $Offer->getSite();

    LogDebug( 'hValues:', $hValues);

    my @ListObjects = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                      @{$hValues->{'ListObjectIDs'}};
    $PortalConfig->hideOffers(\@ListObjects);
}

sub RemoveSubOffers {
    my $self         = shift;
    my ($Servlet)    = @_;

    my $Form         = $Servlet->form;
    my $Offer        = $Servlet->object;
    my $PortalConfig = $Offer->get('PortalConfig');
    my $hValues      = $Form->form($Offer, 'ListedObjects');
    my $Shop         = $Offer->getSite();

    my @ListObjects  = map { $Shop->loadObject($_->{'ListObjectID'}, 'AmazonOffer') }
                      @{$hValues->{'ListObjectIDs'}};

    # delete only if accepted or no confirmation is set
    if ($Servlet->user->get('DeleteConfirmation')
        and not $Servlet->params('ACCEPT')) {
        $Servlet->vars('DELETE',1);
        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @ListObjects]);
        return;
    } else {
        $Servlet->vars('DELETE',0);
    }
    my @OfferIDs = $Servlet->form->value('ListObjectID');
    my $aOffers = $Shop->loadObjects( \@OfferIDs, 'AmazonOffer' );
    $Servlet->testPermission($_, 'Save') foreach @$aOffers;
    $PortalConfig->removeOffers($aOffers);
}

sub AddProductCode {
    my $self = shift;
    my ($Servlet) = @_;

    my $Offer = $Servlet->object;
    my $Form  = $Servlet->form;

    my $ProductCode = $Form->value('ProductCode');

    Readonly::Scalar my $UPC_LENGTH => 12;
    Readonly::Scalar my $EAN_LENGTH => 13;

    $Form->executeFormError({
            'Name'       => 'InvalidProductCode',
            'Reason'     => 'InvalidProductCode',
    }) if ( !defined($ProductCode)
            || (length $ProductCode != $UPC_LENGTH && length $ProductCode != $EAN_LENGTH));

    eval {
        TestUPCEAN('UPC',$ProductCode) if (length $ProductCode == $UPC_LENGTH);
        TestUPCEAN('EAN',$ProductCode) if (length $ProductCode == $EAN_LENGTH);
    };
    if (ExistsError()) {
        $Form->executeFormError({
            'Name'       => 'InvalidProductCode',
            'Reason'     => 'InvalidProductCode',
        });
    }

    my $Product = $Offer->get('Product');
    $Product->set({'UPCEAN' => $ProductCode });

}
1;
