#========================================================================================
# §package      DE_EPAGES::Coupon::API::TLE::CouponHandler
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements TLE functions for coupons.
#========================================================================================
package DE_EPAGES::Coupon::API::TLE::CouponHandler;
use base DE_EPAGES::TLE::API::LoopHandler;

use strict;
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);
use DE_EPAGES::Coupon::DAL::Coupon qw (DAL_GetCouponConditionsByCouponCampaignID); ## no critic qw(DE_EPAGES::Style::RequireCorrectImports)

#========================================================================================
# §function     RegisterHandlerProc
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# §description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler = new DE_EPAGES::Coupon::API::TLE::CouponHandler;
# §example      my $PrefixHandler  = new DE_EPAGES::Coupon::API::TLE::CouponHandler;
#----------------------------------------------------------------------------------------
# §description  create a new handler
#----------------------------------------------------------------------------------------
# §return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;

    $Processor->registerHandler('FunctionHandler', $self, 'GetCouponConditionsByCouponCampaign');

    return;
}

#========================================================================================
# §function     GetCouponConditionsByCouponCampaign
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #GetCouponConditionsByCouponCampaign(#CouponCampaign.ID)
#----------------------------------------------------------------------------------------
# §description  Retrieve coupon condition rows directly from the DB
#----------------------------------------------------------------------------------------
# §input        #CampaignID | object id of the coupon campaign | integer
#========================================================================================
sub GetCouponConditionsByCouponCampaign { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($CampaignID) = @$aParams;

    #-- Optimisation attempt. Store the conditions into the current servlet so that we don't need to run this TLE function again
    #-- if it is called again in the same servlet and thus, template context
    #-- This variable is cleared for each ep6 servlet request. Could we also use a 'session variable'? Probably, but this is, I think, more efficient ...
    my $Servlet;
    eval {
        $Servlet = GetAppServer()->currentServlet();
    };
    $Servlet = undef if $@;
    if ( $Servlet && $Servlet->can('vars') && $Servlet->vars('TCouponConditions') ) {
        return $Servlet->vars('TCouponConditions');
    }

    my $aaConditions = DAL_GetCouponConditionsByCouponCampaignID($CampaignID);

    my @ProductConditions = map { {'ConditionObjectID' => $_->[0]} } grep { $_->[1] eq 'PRODUCT' } @$aaConditions;
    my @ShippingConditions = map { {'ConditionObjectID' => $_->[0]} } grep { $_->[1] eq 'SHIPPING' } @$aaConditions;
    my @PaymentConditions = map { {'ConditionObjectID' => $_->[0]} } grep { $_->[1] eq 'PAYMENT' } @$aaConditions;

    my $hInfo = {};
    $hInfo->{'Total'} = scalar @$aaConditions;
    $hInfo->{'ProductConditions'} = \@ProductConditions;
    $hInfo->{'ShippingConditions'} = \@ShippingConditions;
    $hInfo->{'PaymentConditions'} = \@PaymentConditions;

    #-- Store it in the servlet
    $Servlet->vars('TCouponConditions', $hInfo) if $Servlet && $Servlet->can('vars');

    return $hInfo;
}

1;
