#========================================================================================
# package      DE_EPAGES::Coupon::Hooks::Product
# state        public
# description  On delete product also delete product from table couponcondition.
#========================================================================================
package DE_EPAGES::Coupon::Hooks::Product;

use strict;
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf);
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Coupon::DAL::Coupon qw (DAL_GetCouponConditionsByCouponCampaignID); ## no critic qw(DE_EPAGES::Style::RequireCorrectImports)

#========================================================================================
# function     OnDeleteProduct
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  delete product condition if product is deleted
#               (Hook OBJ_DeleteProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key
#               <ul>
#                   <li>Object - Product - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;

    LoadClassByAlias('CouponCondition')->deleteObjects('ByConditionObjectID', [$hParams->{'ProductID'}]);

    return 1;
}

#========================================================================================
# function     ProductDiscount
#----------------------------------------------------------------------------------------
# syntax       ProductDiscount( $hParams );
#----------------------------------------------------------------------------------------
# description  Calculate discount for product
#               (Hook ProductDiscount)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key
#               <ul>
#                   <li>CurrencyID    - alpha currency code (ISO 4217) - char(3)
#                   <li>TaxModel      - gros or net price    - int
#                   <li>Container     - Container           (optional)- LineItemContainer
#                   <li>LineItem      - LineItemProduct     (optional)- LineItemProduct
#                   <li>User          - user                (optional)- User
#                   <li>Customer      - customer            (optional)- Customer
#                   <li>Object        - Product             - object
#                   <li>ProductID     - product identifier  - integer
#                   <li>Discount      - value for discount  - float
#               </ul> | ref.hash.*
#========================================================================================
sub ProductDiscount {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $LineItem = $hParams->{'LineItem'};
    my $Container = $hParams->{'Container'};
    my $CurrencyID   = $hParams->{'CurrencyID'};
    my $Shop = $Product->getSite();

    return unless defined $LineItem;
    return unless $LineItem->instanceOf('LineItemProduct');
    my $LineItemCoupon = $Container->get('Coupon');
    return unless defined $LineItemCoupon;
    my $CouponCode = $LineItemCoupon->get('CouponCode');
    return unless defined $CouponCode;

    my $CouponCampaign = $CouponCode->get('CouponCampaign');
    return unless $CouponCampaign->get('Type') eq 'PERCENT';
    return unless $CouponCampaign->get('UseCouponOnlyForSelectedProducts');

    # Set discount to each product which is selected as valid product for campaign.
    my $aaConditions      = DAL_GetCouponConditionsByCouponCampaignID($CouponCampaign->id);
    my @ProductConditions = map {$_->[0] } grep {$_->[1] eq 'PRODUCT'} @$aaConditions;

    my $SuperProduct = $Product->get('SuperProduct');
    if (ElementOf($Product->id, @ProductConditions) or (defined $SuperProduct and ElementOf($SuperProduct->id, @ProductConditions))) {
        my $OldDiscount = defined $hParams->{'Discount'} ? $hParams->{'Discount'} : 0;
        my $CouponDiscount = $CouponCampaign->get('Value') / 100 * ($LineItem->get('BasePrice') * $LineItem->get('Quantity') - $OldDiscount);
        my $Discount = $OldDiscount + $CouponDiscount;
        my $LocaleID = defined($Container) ? $Container->get('LocaleID') : $Shop->get('LocaleID');
        my $Locale = $Shop->siteLocale($LocaleID);
        $hParams->{'Discount'} = $Locale->roundMoney($Discount, $CurrencyID);
        $LineItem->set({'CouponDiscount' => $Locale->roundMoney($CouponDiscount, $CurrencyID)});
    }

    return;
}

1;
