#========================================================================================
# package      DE_EPAGES::Customer::UI::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete objects.
#========================================================================================
package DE_EPAGES::Customer::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;

use DE_EPAGES::Database::API::Connection qw (Transaction);
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias);
use DE_EPAGES::Presentation::API::Object::TrayBuilder;
use DE_EPAGES::Search::API::Object::PagerSearch qw (ReplaceWildcards);
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);
use DE_EPAGES::Search::API::Table::SearchSort qw ( ExistsSearchSortBySearchDefinitionID );
use DE_EPAGES::Core::API::Error qw(ExistsError);
use HTTP::Request;
use LWP::UserAgent;
use URI;
use JSON qw(decode_json);
use POSIX;
use List::Util qw (max);

sub JSONSearchCustomersToInsert {
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;
    my $hParam = $Servlet->form->form($Shop, 'JSONSearchCustomersToInsert');
    my $LanguageID = $Servlet->languageID;

    my $SearchName = 'Login';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('Customer'), $SearchName);

    my $EntriesPerPage = $hParam->{'pageEntries'} // 10; ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    my $Pages = $hParam->{'pages'} // 1;

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'PageSize'         => $EntriesPerPage * $Pages,
        'Page'             => max( ceil($hParam->{'currentPage'} / $Pages), 1 ),
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $LanguageID,
        },
        'UriParameters'  => {
           # pager buttons not required
        },
    );

    # Sorting Logic
    my $OrderBy = $hParam->{'orderBy'};
    my $OrderDesc = $hParam->{'orderDesc'};

    # Default sorting is 'CreationDate' descending
    if (defined $OrderBy){
        $Options{'OrderDesc'} = $OrderDesc ? 1 : 0;
    } else {
        $Options{'OrderDesc'} = $OrderDesc ? 0 : 1;
    }

    if (defined $OrderBy ){
        if ($OrderBy eq 'CustomerGroup') {
            $Options{'OrderBy'} = 'CustomerGroup.Alias';
        }
        elsif ($OrderBy eq 'Name') {
            $Options{'OrderBy'} = 'BillingAddress.LastName';
        } else {
            if ( ExistsSearchSortBySearchDefinitionID($SearchDefinition->id, $OrderBy) ){
                $Options{'OrderBy'} = $OrderBy;
            } else {
                $Options{'OrderBy'} = 'CreationDate';
            }
        }
    } else {
        $Options{'OrderBy'} = 'CreationDate';
    }

    if (defined $hParam->{'WildSearchString'}) {
        # EPG-9002: limit the search string
        Readonly::Scalar my $MaxSearchStringLength => 80;
        my $WCSearchString = substr $hParam->{'WildSearchString'}, 0, $MaxSearchStringLength;

        # replace * with %
        $Options{'Parameters'}{'WildSearchString'} = ReplaceWildcards( $WCSearchString );
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    my $Page = $Search->page;
    my $hJSONResults = {};
    my $hPager = $Page->{'Pager'};
    my $LastPage = ceil($hPager->{'CountOfItems'} / $EntriesPerPage);

    if($hPager->{'CountOfItems'} > 0) {
        my $CurrentPage = $hParam->{'currentPage'} // 1;
        $CurrentPage = 1 if ($CurrentPage > $LastPage || $CurrentPage < 0);
        my $PagerSearchPage = ceil($CurrentPage / $Pages);
        my $aFoundObjects = $Search->items($PagerSearchPage);
        my $Formatter = $Shop->siteLocale($Servlet->locale());
        my $Counter = 1;
        my $Pager = ($PagerSearchPage - 1) * $Pages + 1;

        foreach my $FoundObject ( @$aFoundObjects ) {
            my $hAttributes = $FoundObject->get([ 'ObjectID', 'Alias', 'BillingAddress', 'CustomerGroup', 'CreationDate', 'NameOrAlias' ]);
            my $hName = $hAttributes->{'BillingAddress'}->get(['LastName', 'FirstName']);
            my $CustomerGroupName = defined $hAttributes->{'CustomerGroup'} ? $hAttributes->{'CustomerGroup'}->get('NameOrAlias') : '';

            my $hRowData = {
                'ObjectID' => $hAttributes->{'ObjectID'},
                'Alias'    => $hAttributes->{'Alias'},
                'Name'     => $hName->{'LastName'} . ', ' . $hName->{'FirstName'},
                'EMail'    => $hAttributes->{'BillingAddress'}->get('EMail'),
                'CustomerGroup' => $CustomerGroupName,
                'CreationDate'  => $Formatter->format($hAttributes->{'CreationDate'},'date'),
            };

            push @{$hJSONResults->{$Pager}}, $hRowData;

            if ($Counter % $EntriesPerPage == 0) {
                $Pager++;
            }
            $Counter++;
        }
    }

    my $Builder = DE_EPAGES::Presentation::API::Object::TrayBuilder->new({LanguageID => $LanguageID, TranslationPackage => 'DE_EPAGES::Customer' });
    # $Id, [$IsSelectable, $IsSortable, $IsLinkable, $IsRightAligned], $TranslationKey, $TranslationPackage
    $Builder->addColumnHeader('ObjectID', [1, 0, 0, 0], undef)
            ->addColumnHeader('Alias', [0, 1, 0, 0], 'CustomerNoShort')
            ->addColumnHeader('Name', [0, 1, 0, 0], 'Name')
            ->addColumnHeader('EMail', [0, 1, 0, 0], 'EMail')
            ->addColumnHeader('CustomerGroup', [0, 1, 0, 0], 'CustomerGroup')
            ->addColumnHeader('CreationDate', [0, 1, 0, 0], 'CreationDate');

    my $hResponse = {
        'totalNumber' => $hPager->{'CountOfItems'},
        'lastPage' => $LastPage,
        'tableContent' => $hJSONResults,
        'tableHead' => $Builder->getHeaderList(),
        'orderBy' => $hParam->{'orderBy'}, # original requested sorting , not the mapped one in %Options
        'orderDesc' => $Options{'OrderDesc'}
    };

    $self->writeContentOutput($hResponse);
}

sub JSONSearchUsersToInsert {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $Form = $Servlet->form();
    my $LanguageID = $Servlet->languageID();

    my $GetTableHead = $Form->value('GetTableHead');
    $GetTableHead = defined $GetTableHead && !$GetTableHead ? 0 : 1;

    my $SearchName = 'Users';
    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('User'), $SearchName);

    my $EntriesPerPage = $Form->value('pageEntries') // 10; ## no critic (ValuesAndExpressions::ProhibitMagicNumbers)
    my $Pages = $Form->value('pages') // 1;

    my %Options = (
        'Servlet' => $Servlet,
        'Object'  => $Shop,
        'SearchDefinition' => $SearchDefinition,
        'Page'             => max( ceil($Form->value('currentPage') / $Pages), 1 ),
        'PageSize'         => $EntriesPerPage * $Pages,
        'Parameters' => {
            'Shop.ID'    => $Shop->id,
            'LanguageID' => $LanguageID,
            'ExcludeGroupIDs' => [ map { $_->id } @{$Shop->get('NonStorefrontUserGroups')} ],
            'UserClass.ID' => LoadClassByAlias('User')->id,
        },
        'UriParameters'  => {
               # pager buttons not required
        },
    );

    my $OrderBy = $Form->value('orderBy');
    my $OrderDesc = $Form->value('orderDesc');
    # Default sorting is 'CreationDate' descending
    if (defined $OrderBy){
        $Options{'OrderDesc'} = $OrderDesc ? 1 : 0;
    } else {
        $Options{'OrderDesc'} = $OrderDesc ? 0 : 1;
    }

    # Set sorting.
    if (defined $OrderBy) {
        if( ExistsSearchSortBySearchDefinitionID($SearchDefinition->id, $OrderBy) ) {
            $Options{'OrderBy'} = $OrderBy;
        }
        else {
            if ($OrderBy eq 'Name') {
                $Options{'OrderBy'} = 'NameString';
            } elsif ($OrderBy eq 'EMail') {
                $Options{'OrderBy'} = 'EMailString';
            } else {
                $Options{'OrderBy'} = 'CreationDate';
                $OrderBy = 'CreationDate';
            }
        }
    } else {
        $Options{'OrderBy'} = 'CreationDate';
        $OrderBy = 'CreationDate';
    }

    if (defined $Form->value('WildSearchString')) {
        # EPG-9002: limit the search string
        Readonly::Scalar my $MaxSearchStringLength => 80;
        my $WCSearchString = substr $Form->value('WildSearchString'), 0, $MaxSearchStringLength;

        # replace * with %
        $Options{'Parameters'}{'EMailString'} = ReplaceWildcards( $WCSearchString );
    }

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);
    my $Page = $Search->page;

    my $hJSONResults = {};
    my $hPager = $Page->{'Pager'};
    my $LastPage = ceil($hPager->{'CountOfItems'} / $EntriesPerPage);

    if($hPager->{'CountOfItems'} > 0) {
        my $CurrentPage = $Form->value('currentPage') // 1;
        if ($CurrentPage > $LastPage || $CurrentPage < 0) {
            $CurrentPage = 1;
        }
        my $PagerSearchPage = ceil($CurrentPage / $Pages);
        my $aUserList = $Search->items($PagerSearchPage);
        my $Formatter = $Shop->siteLocale($Servlet->locale());
        my $Counter = 1;
        my $Pager = ($PagerSearchPage - 1) * $Pages + 1;

        foreach my $User ( @$aUserList ) {
            my $hAttributes = $User->get([ 'ObjectID', 'Alias', 'Name', 'EMail', 'CreationDate' ]);
            my $aGroups = $User->get('Groups');
            my $GroupString = join(', ', map {$_->get('NameOrAlias', $LanguageID)} @$aGroups);
            my $hRowData = {
                'ObjectID' => int($hAttributes->{'ObjectID'}),
                'Alias' => $hAttributes->{'Alias'},
                'Name' => $hAttributes->{'Name'},
                'EMail' => $hAttributes->{'EMail'},
                'CustomerGroup' => $GroupString,
                'CreationDate' => $Formatter->format($hAttributes->{'CreationDate'},'date_tz'),
            };

            push @{$hJSONResults->{$Pager}}, $hRowData;

            if ($Counter % $EntriesPerPage == 0) {
                $Pager++;
            }
            $Counter++;
        }
    }

    my $Builder = DE_EPAGES::Presentation::API::Object::TrayBuilder->new({LanguageID => $LanguageID, TranslationPackage => 'DE_EPAGES::Customer' });
    # $Id, [$IsSelectable, $IsSortable, $IsLinkable, $IsRightAligned], $TranslationKey, $TranslationPackage
    $Builder->addColumnHeader('ObjectID', [1, 0, 0, 0], undef)
            ->addColumnHeader('Name', [0, 1 , 0, 0], 'Name')
            ->addColumnHeader('EMail', [0, 1 , 0, 0], 'EMail')
            ->addColumnHeader('CustomerGroup', [0, 0, 0, 0], 'CustomerGroup')
            ->addColumnHeader('CreationDate', [0, 1, 0, 0], 'CreationDate');

    my $hResponse = {
        'totalNumber' => $hPager->{'CountOfItems'},
        'lastPage' => $LastPage,
        'tableContent' => $hJSONResults,
        'tableHead' => $Builder->getHeaderList(),
        'orderBy' => $OrderBy,
        'orderDesc' => $Options{'OrderDesc'}
    };

    $self->writeContentOutput($hResponse);
}

sub JSONInsertCustomersAndUsers {
    my $self = shift;
    my ($Servlet) = @_;
    my $Group = $Servlet->object;
    my $Shop = $Group->getSite;

    my @CustomerID = $Servlet->form->value('CustomerIDs');
    my @UserID = $Servlet->form->value('UserIDs');
    my $aCustomers = $Shop->loadObjects(\@CustomerID, 'Customer');

    my @Users;
    foreach my $Customer (@$aCustomers) {
        push @Users, @{$Customer->get('Users')};
    };

    push @Users, @{ $Shop->loadObjects(\@UserID, 'User') };
    my @NewUsers = grep { not $Group->existsMember($_) } @Users;

    Transaction( sub {
        $Group->addMember($_) foreach @NewUsers;
    });

    return;
}

# ec.europa.eu rest API does not have CORS defined, so we need to proxy the request.
sub JSONVerifyVATID{
    my $self = shift;
    my ($Servlet) = @_;
    my $Shop = $Servlet->object;

    my $Form = $Servlet->form;
    my $hParam = $Form->form($Shop, 'JSONVerifyVATID');
    my $VATID = $hParam->{VATID};
    my $CountryCode = $hParam->{CountryCode};

    my $URI = URI->new('https://ec.europa.eu');
    $URI->path_segments('taxation_customs', 'vies', 'rest-api', 'ms', $CountryCode, 'vat', $VATID);

    my $Request = HTTP::Request->new('GET', $URI);

    my $ua = LWP::UserAgent->new;
    $ua->timeout(5);
    my $Response = $ua->request($Request);

    if(! $Response->is_success){
        $Form->executeFormError({
            Name => 'VATID',
            Reason => 'FAILED_TO_FETCH_DATA'
        });
    }
    my $hResponse = undef;
    eval{
        $hResponse = decode_json($Response->decoded_content);
    };
    if(ExistsError){
        $Form->executeFormError({
            Name => 'VATID',
            Reason => 'FAILED_TO_FETCH_DATA'
        });
    }
    $self->writeContentOutput($hResponse);
}

1;
