#========================================================================================
# package      DE_EPAGES::PriceList::API::TLE::ProductHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Implements TLE functions for price lists.
#========================================================================================
package DE_EPAGES::PriceList::API::TLE::ProductHandler;
use base DE_EPAGES::TLE::API::LoopHandler;

use strict;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::PerlTools qw( unique );
use DE_EPAGES::Customer::API::Shop qw( GetCustomerForUser );
use DE_EPAGES::PriceList::API::PriceList qw( GetCustomerPriceLists GetProductPriceList );
use DE_EPAGES::Shop::API::Address qw(IsEUAddress);
use DE_EPAGES::Tax::API::Shop qw( TaxMatrixByEUAddress );

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'ProductHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::ProductHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::ProductHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'ProductHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'LOOPPriceInfoQuantities');
    $Processor->registerHandler('FunctionHandler', $self, 'LOOPValueDiscounts');
    $Processor->registerHandler('FunctionHandler', $self, 'LOOPGlobalValueDiscounts');
    $Processor->registerHandler('FunctionHandler', $self, 'GlobalValueDiscounts');
    $Processor->registerHandler('FunctionHandler', $self, 'ValueDiscounts');
    $Processor->registerHandler('FunctionHandler', $self, 'PriceInfoQuantities');
    $Processor->registerHandler('FunctionHandler', $self, 'HasQuantityPrices');
    $Processor->registerHandler('FunctionHandler', $self, 'HasValueDiscounts');
    return;
}

#========================================================================================
# function     LOOPPriceInfoQuantities
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("LOOPPriceInfoQuantities", #Product, #User, #CurrencyID) template #ENDBLOCK
# example      #BLOCK("LOOPPriceInfoQuantities", #Object, #Session.User.Object, #INPUT.Currency)
#                 #Quantity[integer] #OrderUnit.NameOrAlias :
#                 #IF(#Price)#Price[money]#ENDIF
#                 #IF(#Discount)#Discount[float] %#ENDIF
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Loops the template code for each price in the price list that is
#               applicable fo the given user.<br/>
#               The TLE variables #Quantity, #Price and #Discount can be used within
#               the block.<br/>
#               #Quantity contains the minimum quantity.<br/>
#               #Price contains the price per order unit.<br/>
#               #Discount contains the discount in percent.
#----------------------------------------------------------------------------------------
# input        #Product | product object | object
# input        #User | user object | object
# input        #CurrencyID | currency id (e.g. 'USD') | string
#========================================================================================
sub LOOPPriceInfoQuantities { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;

    return $self->LOOP($Processor,[$self->PriceInfoQuantities($Processor, $aParams)], $cTemplate);
}

#========================================================================================
# function     PriceInfoQuantities
# state        public
#----------------------------------------------------------------------------------------
# syntax       #LOCAL("LocalVariableXY", #PriceInfoQuantities(#Object, #User, #CurrencyID))
# example      #LOCAL("InfoPriceQuantities", #PriceInfoQuantities(#Object,#Session.User.Object, #INPUT.Currency))
#                   #LOOP(#InfoPriceQuantities)
#                       ...
#                   #ENDLOOP
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Loop variable for PriceInfoQuantities
#               #Quantity contains the minimum quantity.<br/>
#               #Price contains the price per order unit.<br/>
#               #ReferencePrice contains the price per reference order unit.<br/>
#               #Discount contains the discount in percent.<br/>
#               #EffectivePrice contains the effective price per reference order unit<br/>
#               #EffectiveDiscount contains the effective discount in percent.
#----------------------------------------------------------------------------------------
# input        #Product | product object | object
# input        #User | user object | object
# input        #CurrencyID | currency id (e.g. 'USD') | string
#========================================================================================
sub PriceInfoQuantities { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Product, $User, $CurrencyID ) = @$aParams;

    my $SuperProduct = $Product->get('SuperProduct');
    $Product = $SuperProduct if defined $SuperProduct and not $Product->get('HasSubOwnPrices');

    my $Shop = $Product->getSite();
    my $Customer = defined $User ? GetCustomerForUser($User, $Shop) : undef;
    my $PriceList = GetProductPriceList($Product, $Customer, $CurrencyID, $Shop, $User);
    return [] unless defined $PriceList;

    # collect scale prices and quantity discounts
    my $TaxModel = defined $Customer ? $Customer->get('TaxModel') : undef;
    $TaxModel = $Shop->get('TaxModel') unless defined $TaxModel;

    my $TaxClassForInfo = undef;
    # only if we have an active delivery threshold regulation
    if((defined $User) && $Shop->get('DeliveryThresholdIsActive')){
        #find the correct destination country based on the Basket/User Billing/ShippingAddress
        # 1st Basket, 2nd User
        # kee im mind that additional shipping addresses are stored below the user and not customer
        # only customer object stores the BillingAddress!
        my $DestinationAddress;
        my $Basket = defined $User->get('Baskets') ? $User->get('Baskets')->[0] : undef;

        if(defined $Basket){
            $DestinationAddress = $Basket->get('ShippingAddress') // $Basket->get('BillingAddress') // undef;
        }
        # get destination address from user if we have no address on the basket
        unless(defined $DestinationAddress){
            $DestinationAddress = $User->get('ShippingAddress') // $User->get('BillingAddress') // undef;
        }

        # get the correct TaxClass
        if($DestinationAddress && IsEUAddress($DestinationAddress)) {
            my $TaxMatrix = TaxMatrixByEUAddress($DestinationAddress);
            $TaxClassForInfo = $Product->getMappedTaxClassByTaxMatrix($TaxMatrix);
        }
    }

    # make sure that the scaleprices are fetched from the product by using the correct taxclass
    my $ahScalePrices = $PriceList->getScalePrices( $Product, $TaxModel, $TaxClassForInfo, undef );
    my $ahQuantityDiscounts = $PriceList->getQuantityDiscounts( $Product, $TaxModel );
    return [] unless scalar @$ahScalePrices or scalar @$ahQuantityDiscounts;

    my %Prices = map { $_->{'Quantity'} => $_->{'Price'} } @$ahScalePrices;
    my %Discounts = map { $_->{'Quantity'} => $_->{'Discount'} } @$ahQuantityDiscounts;
    my @Quantities = sort { $a <=> $b } unique( keys %Prices, keys %Discounts );

    my $EffectivePrice = $Product->getListPrice($CurrencyID, $TaxModel, 0);
    my $EffectiveDiscount = undef;
    my @Loop;
    foreach my $Quantity ( @Quantities ) {
        my $Price = $Prices{$Quantity};
        my $Discount = $Discounts{$Quantity};
        $EffectivePrice = $Price if defined $Price;
        $EffectiveDiscount = $Discount if defined $Discount;
        my $ReferencePrice = $Product->getReferencePrice($CurrencyID, $EffectivePrice, $EffectiveDiscount);

        push @Loop, {
            'Quantity' => $Quantity,
            'Price' => $Price,
            'Discount' => $Discount,
            'EffectivePrice' => $EffectivePrice,
            'EffectiveDiscount' => $EffectiveDiscount,
            'ReferencePrice' => $ReferencePrice,
            'CurrencyID' => $CurrencyID,
        }
    }

    return \@Loop;
}

#========================================================================================
# function     LOOPValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("LOOPValueDiscounts", #Product, #User, #CurrencyID) template #ENDBLOCK
# example      #BLOCK("LOOPValueDiscounts", #Object, #Session.User.Object, #INPUT.Currency)
#                 #Value[money] : #Discount[float]
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Loops the template code for each value discount in the price list that is
#               applicable fo the given user.<br/>
#               The TLE variables #Value and #Discount can be used within
#               the block.<br/>
#               #Value contains the minimum price.<br/>
#               #Discount contains the discount in percent.
#----------------------------------------------------------------------------------------
# input        #Product | product object | object
# input        #User | user object | object
# input        #Currency | currency id (e.g. 'USD') | string
#========================================================================================
sub LOOPValueDiscounts { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;

    return $self->LOOP($Processor, [$self->ValueDiscounts($Processor, $aParams)], $cTemplate);
}

#========================================================================================
# function     ValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       #LOCAL("DiscountValues", #ValueDiscounts(#Object,#Session.User.Object, #INPUT.Currency))
# example      #LOCAL("DiscountValues", #ValueDiscounts(#Object,#Session.User.Object, #INPUT.Currency))
#                   #SET("COUNTValueDiscount", 1)
#                   #IF(#COUNT(#DiscountValues))
#                       ...
#                       #LOOP(#DiscountValues)
#                           ...
#                       #ENDLOOP
#                       ...
#                   #ENDIF
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Loop variable for LOOPValueDiscounts
#               #Quantity contains the minimum quantity.<br/>
#               #Price contains the price per order unit.<br/>
#               #Discount contains the discount in percent.
#----------------------------------------------------------------------------------------
# input        #Product | product object | object
# input        #User | user object | object
# input        #CurrencyID | currency id (e.g. 'USD') | string
#========================================================================================
sub ValueDiscounts { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Product, $User, $CurrencyID) = @$aParams;

    my $SuperProduct = $Product->get('SuperProduct');
    $Product = $SuperProduct if defined $SuperProduct and not $Product->get('HasSubOwnPrices');

    my $Shop = $Product->getSite();
    my $Customer = defined $User ? GetCustomerForUser($User, $Shop) : undef;
    my $PriceList = GetProductPriceList($Product, $Customer, $CurrencyID, $Shop, $User);
    return [] unless defined $PriceList;

    my $TaxModel = defined $Customer ? $Customer->get('TaxModel') : undef;
    $TaxModel = $Shop->get('TaxModel') unless defined $TaxModel;

    return [ sort {$a->{'Value'} <=> $b->{'Value'} }
        @{ $PriceList->getValueDiscounts( $Product, $TaxModel ) } ];
}

#========================================================================================
# function     HasQuantityPrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       #FUNCTION("HasQuantityPrices", #Product, #PriceList)
# example      #IF(#FUNCTION("HasQuantityPrices", #Product.Object, #PriceList.Object))
#                 {HasQuantityPrices}
#               #ENDIF
#----------------------------------------------------------------------------------------
# description  Returns true if a product has any quantity discounts in the given
#               price list.
#----------------------------------------------------------------------------------------
# input        #Product | product object | object
# input        #PriceList | PriceList object | object
#========================================================================================
sub HasQuantityPrices { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($Product, $PriceList ) = @$aParams;
    return ($PriceList->hasScalePrices( $Product ) ||
            $PriceList->hasQuantityDiscounts( $Product ))
        ? 1 : 0;
}

#========================================================================================
# function     HasValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       #FUNCTION("HasValueDiscounts", #Product, #PriceList)
# example      #IF(#FUNCTION("HasValueDiscounts", #Product.Object, #PriceList.Object))
#                 {HasQuantityPrices}
#               #ENDIF
#----------------------------------------------------------------------------------------
# description  Returns true if a product has any value discounts in the given
#               price list.
#----------------------------------------------------------------------------------------
# input        #Product | product object | object
# input        #PriceList | PriceList object | object
#========================================================================================
sub HasValueDiscounts { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;
    my ($Product, $PriceList ) = @$aParams;
    return $PriceList->hasValueDiscounts( $Product );
}

#========================================================================================
# function     LOOPGlobalValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BLOCK("LOOPGlobalValueDiscounts", #Shop, #User, #CurrencyID) template #ENDBLOCK
# example      #BLOCK("LOOPGlobalValueDiscounts", #Shop, #Session.User.Object, #INPUT.Currency)
#                 #Value[money] : #Discount[float]
#               #ENDBLOCK
#----------------------------------------------------------------------------------------
# description  Loops the template code for each value discount in the price list that is
#               applicable fo the given user.<br/>
#               The TLE variables #Value and #Discount can be used within
#               the block.<br/>
#               #Value contains the minimum price.<br/>
#               #Discount contains the discount in percent.
#----------------------------------------------------------------------------------------
# input        #Shop | shop object | object
# input        #User | user object | object
# input        #Currency | currency id (e.g. 'USD') | string
#========================================================================================
sub LOOPGlobalValueDiscounts { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams, $cTemplate) = @_;

    return $self->LOOP($Processor, [$self->GlobalValueDiscounts($Processor, $aParams)], $cTemplate);
}

#========================================================================================
# function     GlobalValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       #LOCAL("GlobalValues", #GlobalValueDiscounts(#Object,#Session.User.Object, #INPUT.Currency))
# example      #LOCAL("GlobalValues", #GlobalValueDiscounts(#Object,#Session.User.Object, #INPUT.Currency))
#                   #SET("COUNTValueDiscount", 1)
#                   #IF(#COUNT(#DiscountValues))
#                       ...
#                       #LOOP(#DiscountValues)
#                           ...
#                       #ENDLOOP
#                       ...
#                   #ENDIF
#               #ENDLOCAL
#----------------------------------------------------------------------------------------
# description  Loop variable for LOOPGlobalValueDiscounts
#               #Value contains the minimum price.<br/>
#               #Discount contains the discount in percent.
#               #DiscountAbsolute the absolute discount.
#               #EffectiveDiscount the effective discount in percent.
#               #EffectiveDiscountAbsolute the effective absolute discount.
#----------------------------------------------------------------------------------------
# input        #Shop | shop object | object
# input        #User | user object | object
# input        #Currency | currency id (e.g. 'USD') | string
#========================================================================================
sub GlobalValueDiscounts { ## no critic qw(RequireCommentForExportedFunctions)
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $User, $CurrencyID ) = @$aParams;

    #pricelist by customer stand in front of array, than follows pricelists by group
    my $aPriceLists = GetCustomerPriceLists(undef, $CurrencyID, $Shop, $User);

    my $EffectiveDiscount = undef;
    my $EffectiveDiscountAbsolute = undef;
    my @Loop;
    foreach my $PriceList (@$aPriceLists) {
        my $aGlobalDiscounts = $PriceList->get('GlobalValueDiscounts');
        next unless scalar @$aGlobalDiscounts;

        foreach my $GlobalValueDiscount ( @$aGlobalDiscounts ) {
            my $Discount = $GlobalValueDiscount->get('Discount');
            my $DiscountAbsolute = $GlobalValueDiscount->get('DiscountAbsolute');
            $EffectiveDiscount = $Discount if defined $Discount;
            $EffectiveDiscountAbsolute = $DiscountAbsolute if defined $DiscountAbsolute;

            push @Loop, {
                'Value' => $GlobalValueDiscount->get('Value'),
                'Discount' => $Discount,
                'DiscountAbsolute' => $DiscountAbsolute,
                'EffectiveDiscount' => $EffectiveDiscount,
                'EffectiveDiscountAbsolute' => $EffectiveDiscountAbsolute
            };
        }

        return \@Loop;
    }

    return [];
}

1;
