#==============================================================================
# package      DE_EPAGES::PriceList::API::WebService::PriceListAssignmentService
# base         DE_EPAGES::Shop::API::WebService::BaseShopService
# state        public
# description  Implements the PriceList Assignment WebService.
#==============================================================================
package DE_EPAGES::PriceList::API::WebService::PriceListAssignmentService;
use base DE_EPAGES::Shop::API::WebService::BaseShopService;

use strict;
use Config;

use DE_EPAGES::Core::API::Log qw(LogDebug);
use DE_EPAGES::Core::API::Error qw (ResetError ExistsError GetError);

#ERRORDEFINITION
sub ErrorDefinition {
    return {}
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::PriceList::API::WebService::PriceListAssignmentService->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    $self->SUPER::new((
        'ServiceUrn' => 'urn://epages.de/WebService/PriceListAssignmentService/2005/01',
        'ServiceTypesUrn' => 'urn://epages.de/WebService/PriceListAssignmentTypes/2005/01',
        @Options,
    ));
}

#========================================================================================
# function     getScalePrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getScalePrices( $aProducts, $aPriceLists )
#----------------------------------------------------------------------------------------
# description  WebService-Method: getInfo
#               Returns information about scale prices for given products and pricelists
#----------------------------------------------------------------------------------------
# input        $aProducts | list of product paths | ref.array.string
# input        $aPriceLists | list of pricelist paths | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given products | SOAP::Data
#========================================================================================
sub getScalePrices {
    my $self = shift;

    # expects a list of product paths and a list of pricelist paths
    my ($aProducts, $aPriceListPaths) = @_;
    LogDebug('aProducts=', $aProducts);
    LogDebug('aPriceListPaths=', $aPriceListPaths);

    my $Shop = $self->site;
    my $InputTaxModel = $Shop->get('InputTaxModel');

    my @soapProducts = ();
    my $result;

    my $aPriceLists = [];
    foreach my $pricelistPath (@$aPriceListPaths) {
        # get object
        push @$aPriceLists, $self->loadShopObject($pricelistPath, 'PriceList');
    }

    foreach my $productPath (@$aProducts) {
        my $soapProduct;

        eval {
            # get object
            my $product= $self->loadShopObject($productPath, 'Product');

            my @soapProductPriceLists = ();
            foreach my $pricelist (@$aPriceLists) {
                my $ahScalePrices =  $pricelist->getScalePrices( $product, $InputTaxModel, undef, 1 );
                LogDebug('scaleprices('.$product->id.','.$pricelist->id.'):', $ahScalePrices);

                my @soapScalePrices;
                push @soapScalePrices, $self->soapData('ScalePrice', {
                    'Quantity'  => $self->soapFloat( $_->{'Quantity'}),
                    'Price'     => $self->soapFloat( $_->{'Price'}),
                }, 'TScalePrice') foreach @$ahScalePrices;

                push @soapProductPriceLists, $self->soapData('PriceList', {
                    'PriceList'     => $self->soapString( $pricelist->get('Path')),
                    'ScalePrices'   => $self->soapList(\@soapScalePrices, 'TScalePrice', $self->getServiceTypesUrn),
                }, 'TPriceListScalePrice');
            }

            my $hOutProduct = {
                'Product' => $self->soapString( $product->get('Path') ),
                'PriceLists' => $self->soapList(\@soapProductPriceLists, 'TPriceListScalePrice', $self->getServiceTypesUrn),
            };

            # build return object
            $soapProduct = $self->soapData('Product', $hOutProduct, 'TGetScalePrices_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProduct = $self->soapData('Product', {
                'Product'     => $self->soapString( $productPath ),
                'Error'       => $self->soapError($Error)
            }, 'TGetScalePrices_Return');
            ResetError();
        }
        push @soapProducts, $soapProduct;
    }

    $result = $self->soapData('Products', $self->soapList(\@soapProducts, 'TGetScalePrices_Return', $self->getServiceTypesUrn) );
    $self->mapType([qw (TGetScalePrices_Return TScalePrice TPriceListScalePrice)], $self->getServiceTypesUrn);
    return $result;
}

#========================================================================================
# function     setScalePrices
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->setScalePrices( $aProducts )
#----------------------------------------------------------------------------------------
# description  WebService-Method: update
#               Updates the scale prices of a list of products/pricelists.
#----------------------------------------------------------------------------------------
# input        $aProducts | list of triples:
#                               <ul>
#                                   <li>Product - object path to product
#                                   <li>PriceList - object path to pricelist,
#                                   <li>ScalePrices - list of scale prices:
#                                       <ul>
#                                           <li>Quantity - float
#                                           <li>Price    - float
#                                       </ul> - ref.hash
#                               </ul> | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given products/pricelists | SOAP::Data
#========================================================================================
sub setScalePrices {
    my $self = shift;

    # expects a list of product paths with according pricelist data
    my ($aProducts) = @_;
    LogDebug('aProducts=', $aProducts);

    my $Shop = $self->site;
    my $InputTaxModel = $Shop->get('InputTaxModel');

    my @soapProducts = ();
    my $result;

    foreach my $hInProduct (@$aProducts) {
        my $soapProduct;
        my $productPath = $hInProduct->{'Product'};
        my $pricelistPath = $hInProduct->{'PriceList'};
        eval {
            $self->checkMandatoryFields($hInProduct, ['Product', 'PriceList', 'ScalePrices']);

            # get object
            my $product   = $self->loadShopObject($productPath, 'Product');
            my $pricelist = $self->loadShopObject($pricelistPath, 'PriceList');

            $pricelist->setScalePrices($product, $InputTaxModel, $hInProduct->{'ScalePrices'});

            my $hOutProduct = {
                'Product'           => $self->soapString( $productPath ),
                'PriceList'         => $self->soapString( $pricelistPath ),
                'updated'           => $self->soapBool( 1 ),
            };

            # build return object
            $soapProduct = $self->soapData('Product', $hOutProduct, 'TSetScalePrices_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProduct = $self->soapData('Product', {
                'Product'     => $self->soapString( $productPath ),
                'PriceList'   => $self->soapString( $pricelistPath ),
                'Error'       => $self->soapError($Error)
            }, 'TSetScalePrices_Return');
            ResetError();
        }
        push @soapProducts, $soapProduct;
    }

    $result = $self->soapData('Products', $self->soapList(\@soapProducts, 'TSetScalePrices_Return', $self->getServiceTypesUrn) );
    $self->mapType([qw (TSetScalePrices_Return)], $self->getServiceTypesUrn);
    return $result;
}

#========================================================================================
# function     getQuantityDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getQuantityDiscounts( $aProducts, $aPriceLists )
#----------------------------------------------------------------------------------------
# description  WebService-Method: getInfo
#               Returns information about quantity discounts
#               for given products and pricelists
#----------------------------------------------------------------------------------------
# input        $aProducts | list of product paths | ref.array.string
# input        $aPriceLists | list of pricelist paths | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given products | SOAP::Data
#========================================================================================
sub getQuantityDiscounts {
    my $self = shift;

    # expects a list of product paths and a list of pricelist paths
    my ($aProducts, $aPriceListPaths) = @_;
    LogDebug('aProducts=', $aProducts);
    LogDebug('aPriceListPaths=', $aPriceListPaths);

    my @soapProducts = ();
    my $result;

    my $aPriceLists = [];
    foreach my $pricelistPath (@$aPriceListPaths) {
        # get object
        push @$aPriceLists, $self->loadShopObject($pricelistPath, 'PriceList');
    }

    foreach my $productPath (@$aProducts) {
        my $soapProduct;

        eval {
            # get object
            my $product= $self->loadShopObject($productPath, 'Product');

            my @soapProductPriceLists = ();
            foreach my $pricelist (@$aPriceLists) {
                my $ahQuantityDiscounts = $pricelist->getQuantityDiscounts( $product );
                LogDebug('quantitydiscounts('.$product->id.','.$pricelist->id.'):', $ahQuantityDiscounts);

                my @soapQuantityDiscounts;
                push @soapQuantityDiscounts, SOAP::Data->type( 'TQuantityDiscount')->name('QuantityDiscount' =>{
                    'Quantity'  => $self->soapFloat( $_->{'Quantity'}),
                    'Discount'  => $self->soapFloat( $_->{'Discount'}),
                }) foreach @$ahQuantityDiscounts;

                push @soapProductPriceLists, SOAP::Data->type( 'TPriceListQuantityDiscount')->name('PriceList' => {
                    'PriceList'         => $self->soapString( $pricelist->get('Path')),
                    'QuantityDiscounts' => $self->soapList(\@soapQuantityDiscounts, 'TQuantityDiscount', $self->getServiceTypesUrn),
                });
            }

            my $hOutProduct = {
                'Product'           => $self->soapString( $product->get('Path') ),
                'PriceLists'        => $self->soapList(\@soapProductPriceLists, 'TPriceListQuantityDiscount', $self->getServiceTypesUrn),
            };

            # build return object
            $soapProduct = $self->soapData('Product', $hOutProduct, 'TGetQuantityDiscounts_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProduct = $self->soapData('Product', {
                'Product'     => $self->soapString( $productPath ),
                'Error'       => $self->soapError($Error)
            }, 'TGetQuantityDiscounts_Return');
            ResetError();
        }
        push @soapProducts, $soapProduct;
    }

    $result = $self->soapData('Products', $self->soapList(\@soapProducts, 'TGetQuantityDiscounts_Return', $self->getServiceTypesUrn) );
    $self->mapType([qw (TGetQuantityDiscounts_Return TPriceListQuantityDiscount TQuantityDiscount)], $self->getServiceTypesUrn);
    return $result;
}

#========================================================================================
# function     setQuantityDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->setQuantityDiscounts( $aProducts )
#----------------------------------------------------------------------------------------
# description  WebService-Method: update
#               Updates the quantity discounts of a list of products/pricelists.
#----------------------------------------------------------------------------------------
# input        $aProducts | list of triples:
#                               <ul>
#                                   <li>Product - object path to product
#                                   <li>PriceList - object path to pricelist,
#                                   <li>QuantityDiscounts - list of quantity discounts:
#                                       <ul>
#                                           <li>Quantity - float
#                                           <li>Discount - float
#                                       </ul> - ref.hash
#                               </ul> | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given products/pricelists | SOAP::Data
#========================================================================================
sub setQuantityDiscounts {
    my $self = shift;

    # expects a list of product paths with according pricelist data
    my ($aProducts) = @_;
    LogDebug('aProducts=', $aProducts);

    my $Shop = $self->site;
    my $InputTaxModel = $Shop->get('InputTaxModel');

    my @soapProducts = ();
    my $result;

    foreach my $hInProduct (@$aProducts) {
        my $soapProduct;
        my $productPath = $hInProduct->{'Product'};
        my $pricelistPath = $hInProduct->{'PriceList'};
        eval {
            $self->checkMandatoryFields($hInProduct, ['Product', 'PriceList', 'QuantityDiscounts']);

            # get object
            my $product   = $self->loadShopObject($productPath, 'Product');
            my $pricelist = $self->loadShopObject($pricelistPath, 'PriceList');

            $pricelist->setQuantityDiscounts($product, $InputTaxModel, $hInProduct->{'QuantityDiscounts'});

            my $hOutProduct = {
                'Product'           => $self->soapString( $productPath ),
                'PriceList'         => $self->soapString( $pricelistPath ),
                'updated'           => $self->soapBool( 1 ),
            };

            # build return object
            $soapProduct = $self->soapData('Product', $hOutProduct, 'TSetQuantityDiscounts_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProduct = $self->soapData('Product', {
                'Product'     => $self->soapString( $productPath ),
                'PriceList'   => $self->soapString( $pricelistPath ),
                'Error'       => $self->soapError($Error)
            }, 'TSetQuantityDiscounts_Return');
            ResetError();
        }
        push @soapProducts, $soapProduct;
    }

    $result = $self->soapData('Products', $self->soapList(\@soapProducts, 'TSetQuantityDiscounts_Return', $self->getServiceTypesUrn) );
    $self->mapType([qw (TSetQuantityDiscounts_Return )], $self->getServiceTypesUrn);
    return $result;
}

#========================================================================================
# function     getValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getValueDiscounts( $aProducts, $aPriceLists )
#----------------------------------------------------------------------------------------
# description  WebService-Method: getInfo
#               Returns information about value discounts
#               for given products and pricelists
#----------------------------------------------------------------------------------------
# input        $aProducts | list of product paths | ref.array.string
# input        $aPriceLists | list of pricelist paths | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given products | SOAP::Data
#========================================================================================
sub getValueDiscounts {
    my $self = shift;

    # expects a list of product paths and a list of pricelist paths
    my ($aProducts, $aPriceListPaths) = @_;
    LogDebug('aProducts=', $aProducts);
    LogDebug('aPriceListPaths=', $aPriceListPaths);

    my $Shop = $self->site;
    my $InputTaxModel = $Shop->get('InputTaxModel');

    my @soapProducts = ();
    my $result;

    my $aPriceLists = [];
    foreach my $pricelistPath (@$aPriceListPaths) {
        # get object
        push @$aPriceLists, $self->loadShopObject($pricelistPath, 'PriceList');
    }

    foreach my $productPath (@$aProducts) {
        my $soapProduct;

        eval {
            # get object
            my $product= $self->loadShopObject($productPath, 'Product');

            my @soapProductPriceLists = ();
            foreach my $pricelist (@$aPriceLists) {
                my $ahValueDiscounts = $pricelist->getValueDiscounts( $product, $InputTaxModel );
                LogDebug('valuediscounts('.$product->id.','.$pricelist->id.'):', $ahValueDiscounts);

                my @soapValueDiscounts;
                push @soapValueDiscounts, SOAP::Data->type( 'TValueDiscount')->name('ValueDiscount' =>{
                    'Value'     => $self->soapFloat($_->{'Value'}),
                    'Discount'  => $self->soapFloat($_->{'Discount'}),
                }) foreach @$ahValueDiscounts;

                push @soapProductPriceLists, $self->soapData('PriceList', {
                    'PriceList'         => $self->soapString( $pricelist->get('Path')),
                    'ValueDiscounts'    => $self->soapList(\@soapValueDiscounts, 'TValueDiscount', $self->getServiceTypesUrn),
                }, 'TPriceListValueDiscount');
            }

            my $hOutProduct = {
                'Product'           => $self->soapString( $product->get('Path') ),
                'PriceLists'        => $self->soapList(\@soapProductPriceLists, 'TPriceListValueDiscount', $self->getServiceTypesUrn),
            };

            # build return object
            $soapProduct = $self->soapData('Product', $hOutProduct, 'TGetValueDiscounts_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProduct = $self->soapData('Product', {
                'Product'     => $self->soapString( $productPath ),
                'Error'       => $self->soapError($Error)
            }, 'TGetValueDiscounts_Return');
            ResetError();
        }
        push @soapProducts, $soapProduct;
    }

    $result = $self->soapData('Products', $self->soapList(\@soapProducts, 'TGetValueDiscounts_Return', $self->getServiceTypesUrn) );
    $self->mapType([qw (TGetValueDiscounts_Return TPriceListValueDiscount TValueDiscount)], $self->getServiceTypesUrn);
    return $result;
}

#========================================================================================
# function     setValueDiscounts
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->setValueDiscounts( $aProducts )
#----------------------------------------------------------------------------------------
# description  WebService-Method: update
#               Updates the value discounts of a list of products/pricelists.
#----------------------------------------------------------------------------------------
# input        $aProducts | list of triples:
#                               <ul>
#                                   <li>Product - object path to product
#                                   <li>PriceList - object path to pricelist,
#                                   <li>ValueDiscounts - list of value discounts:
#                                       <ul>
#                                           <li>Value    - float
#                                           <li>Discount - float
#                                       </ul> - ref.hash
#                               </ul> | ref.array.hash
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given products/pricelists | SOAP::Data
#========================================================================================
sub setValueDiscounts {
    my $self = shift;

    # expects a list of product paths with according pricelist data
    my ($aProducts) = @_;
    LogDebug('aProducts=', $aProducts);

    my $Shop = $self->site;
    my $InputTaxModel = $Shop->get('InputTaxModel');

    my @soapProducts = ();
    my $result;

    foreach my $hInProduct (@$aProducts) {
        my $soapProduct;
        my $productPath = $hInProduct->{'Product'};
        my $pricelistPath = $hInProduct->{'PriceList'};
        eval {
            $self->checkMandatoryFields($hInProduct, ['Product', 'PriceList', 'ValueDiscounts']);

            # get object
            my $product   = $self->loadShopObject($productPath, 'Product');
            my $pricelist = $self->loadShopObject($pricelistPath, 'PriceList');

            $pricelist->setValueDiscounts($product, $InputTaxModel, $hInProduct->{'ValueDiscounts'});

            my $hOutProduct = {
                'Product'           => $self->soapString( $productPath ),
                'PriceList'         => $self->soapString( $pricelistPath ),
                'updated'           => $self->soapBool( 1 ),
            };

            # build return object
            $soapProduct = $self->soapData('Product', $hOutProduct, 'TSetValueDiscounts_Return');
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapProduct = $self->soapData('Product', {
                'Product'     => $self->soapString( $productPath ),
                'PriceList'   => $self->soapString( $pricelistPath ),
                'Error'       => $self->soapError($Error)
            }, 'TSetValueDiscounts_Return');
            ResetError();
        }
        push @soapProducts, $soapProduct;
    }

    $result = $self->soapData('Products', $self->soapList(\@soapProducts, 'TSetValueDiscounts_Return', $self->getServiceTypesUrn) );
    $self->mapType([qw (TSetValueDiscounts_Return)], $self->getServiceTypesUrn);
    return $result;
}

1;
