#==============================================================================
# package      DE_EPAGES::PriceList::UI::Product
# state        private
#==============================================================================
package DE_EPAGES::PriceList::UI::Product;
use base DE_EPAGES::Presentation::UI::Object;

use strict;
use DE_EPAGES::Object::API::Factory qw( Transaction );
use DateTime;
use DE_EPAGES::PriceList::API::PriceList qw ( GetUnityPriceList CreateUnityPriceListForCustomerGroup );
use DE_EPAGES::PriceList::API::Constants qw(UNITY_PRICELIST_PREFIX);
use DE_EPAGES::Core::API::PerlTools qw ( unique );

#==============================================================================
# Event handlers
#==============================================================================
sub ViewPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product   = $Servlet->object;
    my $Shop      = $Product->getSite();

    # display only pricelists where the currency is enabled in the shop
    my %ShopCurrencies = map { $_->{'CurrencyID'} => undef } @{$Shop->get('Currencies')};
    my @DisplayPriceLists = grep { exists $ShopCurrencies{$_->get('CurrencyID')} }
                            @{$Shop->get('PriceLists')};
    $Servlet->vars('DisplayPriceLists', [ map { $_->tleHash } @DisplayPriceLists ]);

    # get selected pricelists or the simplified unity pricelist if available
    # if we don't find a unity pricelist or have an ep6 shop we fallback to SelectedPriceList from the form
    my @PriceListIDs = ();
    if ($Shop->get('IsUnityRegistered')) {
        my $UnityPriceList = GetUnityPriceList($Shop);
        if (defined $UnityPriceList) {
            push @PriceListIDs, $UnityPriceList->id;
        }
    } else {
        @PriceListIDs = grep {defined $_ and $_ ne ''} $Servlet->form->value('SelectedPriceList');
    }

    my @PriceLists;
    if( scalar @PriceListIDs ) {
        foreach my $PriceListID (@PriceListIDs ) {
            my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
            $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');
            push @PriceLists, $PriceList;
        }
    }
    elsif( scalar @DisplayPriceLists >= 2 && !$Shop->get('IsUnityRegistered')) {
        # automatically select the first 2 pricelists
        @PriceLists = @DisplayPriceLists[0, 1];
    }
    elsif( scalar @DisplayPriceLists == 1 && !$Shop->get('IsUnityRegistered')) {
        # automatically select the first pricelist
        @PriceLists = @DisplayPriceLists;
    }
    else {
        # there are no pricelists in the currencies of the shop
        return;
    }

    #set params for selectboxes
    my @SelectedPriceLists;
    foreach my $PriceList ( @PriceLists ) {
        my $PriceListID = $PriceList->id;
        my $Info = $PriceList->tleHash;
        $Info->{'SelectedPriceListID'} = $PriceListID;
        push @SelectedPriceLists,$Info;
    }
    $Servlet->params('SelectedPriceLists',\@SelectedPriceLists );

    # collect scale prices, quantity discounts, value discounts
    my %Prices;
    my $InputTaxModel = $Product->getSite()->get('InputTaxModel');
    foreach my $PriceList (@PriceLists) {
        my $ahScalePrices =  $PriceList->getScalePrices( $Product, $InputTaxModel, undef, 1 );
        $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Price'} = $_->{'Price'} foreach @$ahScalePrices;
        my $ahQuantityDiscounts =  $PriceList->getQuantityDiscounts( $Product, $InputTaxModel, 1 );
        $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Discount'} = $_->{'Discount'} foreach @$ahQuantityDiscounts;
    }

    #scale prices , quantity discounts
    my $Counter = 0;
    my @PriceInfoQuantities = map {
        my $Quantity = $_;
        {
            'Quantity' => $Quantity,
            'PriceInfos' => [
                map {
                    my $PriceList = $_;
                    my $hPrice = $Prices{$Quantity*1}{$PriceList->id} || {};
                    {
                        'PriceListID' => $PriceList->id,
                        'CurrencyID' => $PriceList->get('CurrencyID'),
                        'ErrorItemNo' => $Counter++,
                        'Quantity' => $Quantity,
                        'Price' => $hPrice->{'Price'},
                        'Discount' => $hPrice->{'Discount'},
                    }
                } @PriceLists
            ]
        };
    } sort { $a <=> $b } keys %Prices;
    $Servlet->params('PriceInfoQuantities',\@PriceInfoQuantities );

    #value discounts for the first pricelist
    my $ahValueDiscount = $PriceLists[0]->getValueDiscounts( $Product, $InputTaxModel, 1 );
    $Servlet->params('ValueDiscounts', $ahValueDiscount);

    return;
}

sub UnityViewPriceListPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product   = $Servlet->object;
    my $Shop      = $Product->getSite();
    my $hInfoShop = $Shop->get([qw(InputTaxModel CurrencyID PriceLists)]);

    # Collect all Unity pricelist ids except the simplified pricelist for bulkprices
    my @PriceListIDs = ();
    my $ahUnityPriceLists = $hInfoShop->{'PriceLists'};
    foreach my $UnityPriceList (@$ahUnityPriceLists) {
        # do not add simplified pricelist for bulkprices to @PriceListIDs
        next if $UnityPriceList->alias eq UNITY_PRICELIST_PREFIX.$hInfoShop->{'CurrencyID'};
        push @PriceListIDs, $UnityPriceList->id;
    }

    my @PriceLists;
    if( scalar @PriceListIDs ) {
        foreach my $PriceListID (@PriceListIDs ) {
            my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
            $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');
            push @PriceLists, $PriceList;
        }
    }
    else {
        # there are no pricelists in the shop
        return;
    }

    # collect quantity discounts and scaleprices
    my $ahPrices = [];
    my $ahPriceListQuantityExists = [];
    my $InputTaxModel = $hInfoShop->{'InputTaxModel'};
    foreach my $PriceList (@PriceLists) {
        my $ahScalePrices =  $PriceList->getScalePrices( $Product, $InputTaxModel, undef, 1 );
        my $PriceListCurrencyID = $PriceList->get('CurrencyID');
        my $ahQuantityDiscounts =  $PriceList->getQuantityDiscounts( $Product, $InputTaxModel, 1 );
        my $aQuantities = [];

        #get all quantities from scaleprices and quantitydiscounts for a pricelist
        foreach my $ScalePriceQuantity (@$ahScalePrices) {
            push @$aQuantities, $ScalePriceQuantity->{'Quantity'};
        }

        foreach my $QuantityDiscountQuantity (@$ahQuantityDiscounts) {
            push @$aQuantities, $QuantityDiscountQuantity->{'Quantity'};
        }

        #remove duplicate quantities
        @$aQuantities = unique @$aQuantities;

        my $QuantityExists = 0;
        foreach my $Quantity (@$aQuantities) {
            my $ScalePricePrice;
            my $QuantityDiscountValue;
            foreach my $ScalePrice (@$ahScalePrices) {
                if ($Quantity eq $ScalePrice->{'Quantity'}) {
                    $ScalePricePrice = $ScalePrice->{'Price'};
                }
            }

            foreach my $QuantityDiscount (@$ahQuantityDiscounts) {
                if ($Quantity eq $QuantityDiscount->{'Quantity'}) {
                    $QuantityDiscountValue = $QuantityDiscount->{'Discount'};
                }
            }

            push (@$ahPrices, {
                'PriceListID' => $PriceList->id,
                'CurrencyID'  => $PriceListCurrencyID,
                'Price'       => $ScalePricePrice,
                'Quantity'    => $Quantity,
                'Discount'    => $QuantityDiscountValue
            });

            $QuantityExists = 1 unless $QuantityExists;
        }

        my $CustomerGroupID;
        foreach my $Group (@{$Shop->get('CustomerGroups')}) {
            my $aAssignedPriceList = $Group->get('PriceLists');
            next unless (defined $aAssignedPriceList  and scalar(@{$aAssignedPriceList }));
            # Unity shops only have 1 priceList assigned to a customergroup so take first element by default
            if ($PriceList->alias eq @{$aAssignedPriceList}[0]->alias) {
                $CustomerGroupID = $Group->id;
                last;
            }
        }

        push (@$ahPriceListQuantityExists, {
            'PriceListID'    => $PriceList->id,
            'QuantityExists' => $QuantityExists,
            'CustomerGroupID'  => $CustomerGroupID
        });
    }

    my $ahSortedCustomerGroups = [];
    foreach my $CustomerGroup (sort  { $b->{'QuantityExists'} <=> $a->{'QuantityExists'} } @$ahPriceListQuantityExists) {
        push(@$ahSortedCustomerGroups,  $Shop->loadObject($CustomerGroup->{'CustomerGroupID'}, 'CustomerGroup')
        );
    }
    $Servlet->params('SortedCustomerGroups',[ map { $_->tleHash } @$ahSortedCustomerGroups ] );

    my $Counter = 0;
    my @PriceInfoQuantities = map {
        {
                    'PriceListID' => $_->{'PriceListID'},
                    'Quantity'    => $_->{'Quantity'},
                    'PriceInfos'  => [
                        {
                            'CurrencyID'  => $_->{'CurrencyID'},
                            'ErrorItemNo' => $Counter++,
                            'Quantity'    => $_->{'Quantity'},
                            'Price'       => $_->{'Price'},
                            'Discount'    => $_->{'Discount'}
                        }
                    ]
                }
        } sort {$a->{'Quantity'} <=> $b->{'Quantity'}} @$ahPrices;

    # collect scale prices, quantity discounts
    $Servlet->params('PriceInfoQuantities',\@PriceInfoQuantities );

    return;
}

sub SaveQuantityPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    my $ahNewQuantityPrices = [];

    # collect price lists, check permissions, create unity pricelist/customergroup if necessary
    my $hFormValues;
    my %PriceLists;
    if ($Shop->get('IsUnityRegistered')) {
        $hFormValues = $Servlet->form->form($Product, 'UnitySaveQuantityPrices', 1);
        my $ViewAction = $Servlet->params('ViewAction');
        my $ShopCurrency = $Shop->get('CurrencyID');
        my $ahUnityNewQuantityPrices = [];

        # when showing the bulkprices page in unity, we only use the simplyfied pricelist
        if ($ViewAction eq 'UnityMBO-ViewPrices'){
            my $PriceList = GetUnityPriceList($Shop, 1);
            foreach my $NewQuantityPrice (@{$hFormValues->{'NewQuantityPrices'}}) {
                push(@$ahUnityNewQuantityPrices, {
                    'NewDiscount'    => $NewQuantityPrice->{'NewDisscount'},
                    'NewPriceListID' => $PriceList->id,
                    'NewPrice'       => $NewQuantityPrice->{'NewPrice'},
                    'NewQuantity'    => $NewQuantityPrice->{'NewQuantity'}
                });
            }
            $_->{'PriceListID'} = $PriceList->id for @{$hFormValues->{'QuantityPrices'}};
            $_->{'NewPriceListID'} = $PriceList->id for  @{$ahUnityNewQuantityPrices};

            $PriceLists{$_} = $Shop->loadObject($_, 'PriceList') foreach map { $_->{'PriceListID'} } @{$hFormValues->{'QuantityPrices'}};
            $PriceLists{$_} = $Shop->loadObject($_, 'PriceList') foreach map { $_->{'NewPriceListID'} } @{$ahUnityNewQuantityPrices};

        } else {
            # In case we have Unity CustomerGroups without pricelists, we need first to create a new pricelist for that group
            foreach my $NewQuantityPrice (@{$hFormValues->{'NewQuantityPrices'}}){
                my $NewUnityPriceListID;
                if (!defined $NewQuantityPrice->{'NewPriceListID'}) {
                    my $CustomerGroup = $Shop->loadObject($NewQuantityPrice->{'CustomerGroupID'}, 'CustomerGroup');
                    my $NewUnityPriceList = CreateUnityPriceListForCustomerGroup($CustomerGroup);
                    $NewUnityPriceListID = $NewUnityPriceList->id;
                } else {
                    $NewUnityPriceListID = $NewQuantityPrice->{'NewPriceListID'};
                }
                push(@$ahUnityNewQuantityPrices, {
                    'NewDiscount' => $NewQuantityPrice->{'NewDiscount'},
                    'NewPriceListID' => $NewUnityPriceListID,
                    'NewPrice' => $NewQuantityPrice->{'NewPrice'},
                    'NewQuantity' => $NewQuantityPrice->{'NewQuantity'}
                });
            }

            my $ahUnityPriceLists = $Shop->get('PriceLists');
            foreach my $UnityPriceList (@$ahUnityPriceLists) {
                next if $UnityPriceList->alias eq UNITY_PRICELIST_PREFIX.$ShopCurrency;

                $UnityPriceList->{'PriceListID'} = $UnityPriceList->id for @{$hFormValues->{'QuantityPrices'}};
                $UnityPriceList->{'NewPriceListID'} = $UnityPriceList->id for @{$ahUnityNewQuantityPrices};

                my $UnityPriceListObject =  $Shop->loadObject($UnityPriceList->id, 'PriceList');

                foreach my $QuantityPricesPriceListID (@{$hFormValues->{'QuantityPrices'}}) {
                    if ($QuantityPricesPriceListID->{'PriceListID'} eq $UnityPriceList->id) {
                        $PriceLists{$UnityPriceList->id} = $UnityPriceListObject;
                    }
                }
                foreach my $NewQuantityPricesPriceListID (@{$ahUnityNewQuantityPrices}) {
                    if ($NewQuantityPricesPriceListID->{'NewPriceListID'} eq $UnityPriceList->id) {
                        $PriceLists{$UnityPriceList->id} = $UnityPriceListObject;
                    }
                }
            }
        }
        # save value of $ahUnityNewQuantityPrices to new array for later use
        $ahNewQuantityPrices = $ahUnityNewQuantityPrices;
    } else {
        $hFormValues = $Servlet->form->form($Product, 'SaveQuantityPrices');
        $PriceLists{$_} = $Shop->loadObject($_, 'PriceList') foreach map { $_->{'PriceListID'} } @{$hFormValues->{'QuantityPrices'}};
        $PriceLists{$_} = $Shop->loadObject($_, 'PriceList') foreach map { $_->{'NewPriceListID'} } @{$hFormValues->{'NewQuantityPrices'}};

        # save value of $hFormValues->{'NewQuantityPrices'} to new array for later use
        $ahNewQuantityPrices = $hFormValues->{'NewQuantityPrices'};
    }
    $Servlet->testPermission($_, 'MBO-ViewGeneral') foreach values %PriceLists;

    # initialize hashes with empty lists for each pricelist
    # ( $PriceListID => [ {}, {}, ... ], ... )
    my %ScalePrices = map { $_ => [] } keys %PriceLists;
    my %QuantityDiscounts = map { $_ => [] } keys %PriceLists;

    # get existing prices and discounts
    foreach my $hEntry ( @{$hFormValues->{'QuantityPrices'}} ) {
        next unless defined $hEntry->{'Quantity'};
        if( defined $hEntry->{'Price'} ) {
            push @{$ScalePrices{$hEntry->{'PriceListID'}}}, {
                'Quantity' => $hEntry->{'Quantity'},
                'Price' => $hEntry->{'Price'},
            };
        }
        if( defined $hEntry->{'Discount'} ) {
            push @{$QuantityDiscounts{$hEntry->{'PriceListID'}}}, {
                'Quantity' => $hEntry->{'Quantity'},
                'Discount' => $hEntry->{'Discount'},
            };
        }
    }

    # verify new scales prices or quantity discounts
    $self->_validateQuantities($Servlet, $hFormValues, \%ScalePrices, \%QuantityDiscounts);
    # add new scale prices
    my $Quantity = exists $hFormValues->{'NewQuantity'} ? $hFormValues->{'NewQuantity'} : undef;
    foreach my $hEntry ( @{$ahNewQuantityPrices} ) {
        $Quantity = exists $hEntry->{'NewQuantity'} ? $hEntry->{'NewQuantity'} : $Quantity;
        if (defined $Quantity) {
            my $PriceListID = $hEntry->{'NewPriceListID'};
            if (defined $hEntry->{'NewPrice'}) {
                push @{$ScalePrices{$PriceListID}}, {
                    'Quantity' => $Quantity,
                    'Price'    => $hEntry->{'NewPrice'},
                };
            }
            if (defined $hEntry->{'NewDiscount'}) {
                push @{$QuantityDiscounts{$PriceListID}}, {
                    'Quantity' => $Quantity,
                    'Discount' => $hEntry->{'NewDiscount'},
                };
            }
        }
    }

    # update scale prices and quantity discounts
    my $InputTaxModel = $Product->getSite()->get('InputTaxModel');
    foreach my $PriceListID (keys %PriceLists) {
        my $PriceList = $PriceLists{$PriceListID};

        my $ahScalePrices = $ScalePrices{$PriceListID};
        $PriceList->setScalePrices($Product, $InputTaxModel, $ahScalePrices);

        my $ahQuantityDiscounts = $QuantityDiscounts{$PriceListID};
        $PriceList->setQuantityDiscounts($Product, $InputTaxModel, $ahQuantityDiscounts);
    }
    return;
}

# note that for ep6 $QuantityCount{$PriceListID}{*} will normally never exceed a count of 1
# this is just used for unity, where multiple entries for a single pricelists are possible
sub _validateQuantities {
    my $self = shift;
    my ($Servlet, $hFormValues, $ScalePrices, $QuantityDiscounts) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();
    my $Index = 0;
    my %QuantityCount = ( 'ScalePrice' => {}, 'QuantityDiscount' => {});
    my $Quantity = exists $hFormValues->{'NewQuantity'} ? $hFormValues->{'NewQuantity'} : undef;

    foreach my $hEntry ( @{$hFormValues->{'NewQuantityPrices'}} ) {
        $Quantity = exists $hEntry->{'NewQuantity'} ? $hEntry->{'NewQuantity'} : $Quantity;
        if (defined $Quantity) {
            $Quantity = $Product->correctQuantity($Quantity);
            my $PriceListID = $hEntry->{'NewPriceListID'};
            $QuantityCount{'ScalePrice'}{$Quantity}++;
            $QuantityCount{'QuantityDiscount'}{$Quantity}++;
            # test for existing interval
            if ((scalar(grep {$_->{'Quantity'} == $Quantity} @{$ScalePrices->{$PriceListID}})
                 + scalar(grep {$_->{'Quantity'} == $Quantity} @{$QuantityDiscounts->{$PriceListID}})
                 > 0) || $QuantityCount{$PriceListID}{'ScalePrice'}{$Quantity}>1 ||
                $QuantityCount{$PriceListID}{'QuantityDiscount'}{$Quantity}>1 )
            {
                my $hErrorValues = {
                    'Reason'   => 'INTERVAL_EXISTS',
                    'Name'     => 'NewQuantity',
                    'Form'     => 'SaveQuantityPrices',
                    'PriceListID' => $PriceListID,
                    'Quantity' => $Quantity,
                };
                if ($Shop->get('IsUnityRegistered')) {
                    @$hErrorValues{ qw(Form LoopName Index) } = ('UnitySaveQuantityPrices', 'NewQuantityPrices', $Index);
                    $hEntry->{'NewQuantity'} = $Quantity;
                } else {
                    $hFormValues->{'NewQuantity'} = $Quantity;
                }
                $Servlet->form->addFormError($hErrorValues);
            }
            $Index++;
        }
    }
    $Servlet->form->executeFormError();
}

sub DeleteQuantityPrices {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    my $hFormValues  = $Servlet->form->form($Product, 'DeleteQuantityPrices');
    my @DeleteQuantities =  map { $_->{'DeleteQuantity'} } @{$hFormValues->{'QuantityPrices'}};
    return unless scalar @DeleteQuantities;

    # get price lists and check permissions
    my @PriceLists = map { $Shop->loadObject($_, 'PriceList') }
        grep { defined $_ && $_ ne '' }
        map { $_->{'SelectedPriceList'} }
        @{$hFormValues->{'PriceLists'}};
    $Servlet->testPermission($_, 'MBO-ViewGeneral') foreach @PriceLists;
    return unless scalar @PriceLists;


    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template

        # collect scale prices, quantity discounts, value discounts
        my %Prices;
        my $InputTaxModel = $Product->getSite()->get('InputTaxModel');
        foreach my $PriceList (@PriceLists) {
            my $ahScalePrices =  $PriceList->getScalePrices( $Product, $InputTaxModel, undef, 1 );
            $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Price'} = $_->{'Price'} foreach @$ahScalePrices;
            my $ahQuantityDiscounts =  $PriceList->getQuantityDiscounts( $Product, $InputTaxModel, 1 );
            $Prices{$_->{'Quantity'}*1}{$PriceList->id}{'Discount'} = $_->{'Discount'} foreach @$ahQuantityDiscounts;
        }

        my @DeleteObjects = map {
            my $Quantity = $_;
            {
                'Quantity' => $Quantity,
                'PriceInfos' => [
                    map {
                        my $PriceList = $_;
                        my $hPrice = $Prices{$Quantity*1}{$PriceList->id} || {};
                        {
                            'CurrencyID' => $PriceList->get('CurrencyID'),
                            'Price' => $hPrice->{'Price'},
                            'Discount' => $hPrice->{'Discount'},
                        }
                    } @PriceLists
                ]
            };
        } @DeleteQuantities;
        $Servlet->vars('DeleteQuantities',\@DeleteObjects);
        $Servlet->vars('DELETE',1);
        return;
    }
    else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    my %DeleteQuantities = map { $_*1 => 1 } @DeleteQuantities;
    foreach my $PriceList (@PriceLists) {
        $_->delete foreach
            grep { exists $DeleteQuantities{$_->get('Quantity')*1} }
            @{ $PriceList->scalePrices( $Product ) };

        $_->delete foreach
            grep { exists $DeleteQuantities{$_->get('Quantity')*1} }
            @{ $PriceList->quantityDiscounts( $Product ) };
    }

    return;
}

sub SaveValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    # get value discounts
    my $hFormValues = $Servlet->form->form($Product, 'SaveValueDiscounts');
    my $PriceListID = $hFormValues->{'PriceListID'};
    return unless defined $PriceListID;

    my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
    $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');

    my @ValueDiscounts = grep { defined $_->{Value} && defined $_->{Discount} }
        @{$hFormValues->{'ValueDiscounts'}};

    # new value discounts, check for duplicates
    my $NewDiscount = $hFormValues->{'NewDiscount'};
    my $NewValue = $hFormValues->{'NewValue'};
    if (defined $NewValue && defined $NewDiscount) {
        #test for existing interval
        if( scalar grep { $_->{'Value'} == $NewValue } @ValueDiscounts ) {
            $Servlet->form->executeFormError({
                'Reason'=> 'INTERVAL_EXISTS',
                'Name'  => 'NewValue',
                'Form'  => 'NewValueDiscount',
            });
        }

        # add new value to existing
        push @ValueDiscounts, {
            'Value'       => $NewValue,
            'Discount'    => $NewDiscount,
        };
    }

    # save value discounts
    my $InputTaxModel = $Shop->get('InputTaxModel');
    $PriceList->setValueDiscounts($Product, $InputTaxModel, \@ValueDiscounts);

    return;
}

sub DeleteValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $Shop    = $Product->getSite();

    my $hFormValues  = $Servlet->form->form($Product, 'DeleteValueDiscounts');
    my $PriceListID = $hFormValues->{'PriceListID'};
    return unless defined $PriceListID;

    my $PriceList = $Shop->loadObject($PriceListID, 'PriceList');
    $Servlet->testPermission($PriceList, 'MBO-ViewGeneral');

    my $InputTaxModel = $Shop->get('InputTaxModel');
    my $ahValueDiscounts = $PriceList->getValueDiscounts( $Product, $InputTaxModel, 1 );
    my %DeleteValues = map { $_->{'DeleteValue'}*1 => 1 } @{$hFormValues->{'ValueDiscounts'}};

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        my @DeleteObjects = grep { exists $DeleteValues{$_->{'Value'}*1} } @$ahValueDiscounts;
        $Servlet->vars('DeleteObjects', \@DeleteObjects);
        $Servlet->vars('DELETE',1);
    }
    else {
        # find the remaining value discounts
        my @ValueDiscounts = grep { !exists $DeleteValues{$_->{'Value'}*1} } @$ahValueDiscounts;
        Transaction( sub {
            $PriceList->setValueDiscounts( $Product, $InputTaxModel, \@ValueDiscounts )
        } );

        $Servlet->vars('DELETE',0);
    }

    return;
}

1;
